/***************************************************************************
 *            bitmap.h
 *
 *  Wed Aug 24 13:04:52 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __BITMAP_H__
#define __BITMAP_H__

#ifdef HAVE_STDLIB_H
#undef HAVE_STDLIB_H
#endif
#ifdef HAVE_STDDEF_H
#undef HAVE_STDDEF_H
#endif
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <wx/wx.h>
#include <time.h>
#include "exif.h"
#include "types.h"

struct Rgb {
	float b, g, r, a;
	unsigned toRGB32() const;
	void CreateFrom(unsigned);
};

enum ExifHandling {
	RETAIN_EXIF,
	DONT_RETAIN_EXIF,
	DEFAULT
};

class RawImg {
	int x, y;
	int original_x, original_y;
	int rotation;
	int resolution;
	time_t exif_datetime;
	Rgb *data;
	ExifWrapper *exif;

	unsigned char* CreateAlphaPreview(float opacity = 1.0f);
public:
	RawImg();
	~RawImg();
	bool LoadFromJPG(const char *);
	bool LoadFromBMP(const char *);
	bool LoadFromImageFile(const char *);
	bool AutoLoad();
	bool SaveToJPG  (const char *, int quality, 
		bool CreateFrame = false, unsigned FrameColor = 0xffffff,
		float opacity = 1.0f, float gamma = 1.0f, ExifHandling = RETAIN_EXIF);
	bool SaveToBMP (const char *);
	bool SaveToPNG (const char *, bool CreateFrame = false,
		float opacity = 1.0f, float gamma = 1.0f);
	bool SaveToImageFile(const char *, int quality);
	void Destroy();
	bool Resize(int new_x, int new_y);
	unsigned char* CreateRGBPreview(bool CreateFrame = false, 
		unsigned BackgroundColor = 0xffffff, float opacity = 1.0f, 
		float gamma = 1.0f);
	bool ExtractExifDateTime();
	time_t ExifDateTime() const;
	bool GetFullExifInfo(FullExifData & data);

	int get_x() const;
	int get_y() const;
	int get_original_x() const;
	int get_original_y() const;
	int get_size() const;
	int get_rotation() const;
	void set_rotation(int);
	Rgb * get_data() const;
	int get_translated_index(int , int ) const;

	void get_rotated_sizes(int & xsize, int & ysize) const;
	void get_original_rotated_sizes(int & xsize, int & ysize) const;
	
	void set_resolution(int new_res);
	int get_resolution() const;
	ExifWrapper* get_exif() const;

	
	// public data:
	wxString FileName;
	wxString outFileName;

};

typedef struct {
int fs;       // filesize
int lzero;
int bfImgOffset;  // basic header size
}BMHEADER;

typedef struct {
int ihdrsize; 	// info header size
int x,y;      	// image dimensions
Uint16 channels;// number of planes
Uint16 bitsperpixel;
int compression; // 0 = no compression
int biSizeImage; // used for compression; otherwise 0
int pixPerMeterX, pixPerMeterY; // dots per meter settings (why meters and not inches... maybe an attempt to conform to SI? :)
int colors;	 // number of used colors. If all specified by the bitsize are used, then it should be 0
int colorsImportant; // number of "important" colors (All in wonder..)
}
#ifdef __GNUC__
__attribute__((packed))
#endif
BMiHEADER;



#endif // __BITMAP_H__
