/***************************************************************************
 *            project.h
 *
 *  Wed Aug 24 12:48:46 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __PROJECT_H__
#define __PROJECT_H__
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <wx/wx.h>

#include "bitmap.h"
#include "tools.h"

enum ThumbFormatType {
	FORMAT_JPEG,
	FORMAT_PNG
};

enum BmpHandlingType {
	BMP_HANDLING_UNDEFINED,
	BMP_HANDLING_IGNORE,
	BMP_HANDLING_TO_JPG,
	BMP_HANDLING_TO_BMP
};

enum RolloverState {
	NONE,
	HIGHLIGHT,
	CHANGE_OPACITY,
	HIGHLIGHT_AND_OPACITY,
};

enum GalleryType {
	GT_PAGE_WITH_THUMBS,
	GT_ONE_PICTURE_PER_PAGE,
};

#define PICINFO_FILENAME		0x0001
#define PICINFO_DIMENSIONS		0x0002
#define PICINFO_FILESIZE		0x0004

struct ProjectInfo {
	wxString Title, iPath, oPath;
	int PicSize, ThumbSize;
	bool has_frame;
	ThumbFormatType ThumbFormat;
	unsigned BackgroundColor, TextColor;
	unsigned PicsPerRow, PicsPerGallery;
	unsigned InfoPerImg;
	int JpegQuality;
	wxString BackgroundImage;
	RolloverState Rollover;
	BmpHandlingType BmpHandling;
	GalleryType galleryType;
	wxString font;
	bool exif_stats;
	
	void Defaults(void);
};

class ProjectIterator;
class Project {
	
	int count, current;
	ProjectInfo pi;
	RawImg *thumbs;
	bool *state;
	bool have_exif_stats;
	SharedCounter counter, doneCounter;
	wxString globDirPath;
	friend class DoWorkThread;
	friend class ProjectIterator;
	bool RecreateThumbs_implementation(bool master_thread);
	bool CreateGallery_init(void);
	bool CreateGallery_implementation(bool full, bool master_thread);
	bool CreateHtmlPage(int, int);
	bool CreateJavaScriptPage(void);
	void CheckExifStats(void);
	FILE* CreateExifStatsPageHeader(void);
	void AddExposureData(FILE*, Rational *, int);
	void AddApertureData(FILE*, float *, int);
	void AddISOData(FILE*, int *, int);
	void AddFocalLengthData(FILE *, int *, int, bool);
	void WriteJavaScript(FILE *);
	void PutGallerySelector(FILE *, int, int);
	bool NeedsRegeneration(RawImg&, int, bool, int forcedPicSize = 0);
	wxString MakeBMPH(wxString);
	wxString MakeExt(wxString );
	
public:
	Project ();
	~Project();
	void Destroy();
	void FreeImageData();
	bool NewProject (const ProjectInfo &);
	bool SaveToFile(const wxString &);
	bool LoadFromFile(const wxString &, 
                      bool recreate_thumbs = true, 
                      bool guess_orientations = false);
	bool RecreateThumbs(void);
	void SetCurrent(int);
	void AdjustCurrent(int);

	bool CreateGallery(bool);
	bool CreateHtml(void);

	bool RunThread(int);
	
	
	RawImg *GetCurrentImg() const;
	bool GetCurrentState() const;
	void SetCurrentState(bool Include);

	int GetCurrentID() const;
	int size() const;
	int GetCount() const;
	wxString Get_iPath() const;
	void SetProjectInfo(const ProjectInfo &);
	ProjectInfo GetProjectInfo() const;
	bool GetHasFrame() const;
	void SetHasFrame(bool);
	bool MustSave;
	
	const ProjectIterator begin();
	const ProjectIterator end();

	wxString FilenameToSave;
};

class ProjectIterator {
	int current;
	Project *prj;
public:
	ProjectIterator();
	ProjectIterator(int, Project*);
	RawImg* operator ++(int);
	void operator =(const ProjectIterator &);
	bool operator ==(const ProjectIterator &);
	bool operator !=(const ProjectIterator &);
	void operator +=(int);
	RawImg* operator *();
};

enum thread_commands {
	TRD_RECREATE_THUMBS,
	TRD_GENERATE_GALLERY_TRUE,
	TRD_GENERATE_GALLERY_FALSE
};

class DoWorkThread : public wxThread {
	Project *prj;
	SharedCounter *counter;
	int exec_fn;
public:
	DoWorkThread(Project *, int, SharedCounter*);
	virtual void * Entry();
};


#endif // __PROJECT_H__
