/***************************************************************************
 *            tools.h
 *
 *  Tue Aug 23 18:45:10 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __TOOLS_H__
#define __TOOLS_H__


#ifdef _MSC_VER
#define ROUND(x) (int(x))
#else
#define ROUND(x) (lrint(x))
#endif

#include "bitmap.h"

class Project;

int EndX(wxWindow *w);
int EndY(wxWindow *w);
int IMax(int a, int b);
wxString PathLastSlashStrip(wxString);
bool path_is_ok(const wxString &);
bool parse_pair(const char * line, char * field, char * value);
bool ChooseFileName(Project*);
// returns the background color of the object in RGB format (i.e. Blue in LSB)
unsigned BgColorRGB(const wxWindow *);
wxString GetFn(const wxString &);

wxString SlashExtend(wxString);
wxString Capitalize(wxString);
bool ExtensionIs(wxString file, wxString ext);
int GetHex(const wxString &, int );

/* returns the color, located in `where' between color1 and color2
 * For example
 * where = 0.0 - returns color1
 * where = 0.25 - returns a blend of 75% color1 and 25% color2
 * where = 1.0 - return color2
 *
 * `where' can be outside the usual range 0.0-1.0, where extrapolation occurs
 *         but using this is not advisable
 */
unsigned Interpolate(unsigned color1, float where, unsigned color2);

unsigned GetFileSizeInKB(const wxString& fn);

void scale_sizes(int &, int &, int);

bool myCopyFile(const char *InputFile, const char *OutputFile);

wxString stripExtension(wxString);

Uint32 ByteSwap32(Uint32);
Uint16 ByteSwap16(Uint16);

void *sse_malloc(size_t size);
void sse_free(void *);

#if __BYTE_ORDER == __LITTLE_ENDIAN
#	define itohl(x) (x)
#	define itohs(x) (x)
#	define htoil(x) (x)
#	define htois(x) (x)
#else
#	define itohl(x) ByteSwap32(x)
#	define itohs(x) ByteSwap16(x)
#	define htoil(x) ByteSwap32(x)
#	define htois(x) ByteSwap16(x)
#endif

int getCPUCount(void);

/**
 * @brief calculates positions and sizes for an array of buttons in a Frame/Dialog
 *
 * The calculated positions/sizes will be in the bottom-hand side of the
 * frame/dialog, centered, with 10 pix clearance between buttons
 *
 * @param wnd - the target window (used to determine Dialog size)
 * @param p   - [out] - the positions of the buttons
 * @param s   - [out] - the sizes of buttons (will probably be all identical)
 * @param count - the desired number of buttons
 */
void CalculateArrangement(wxWindow *wnd, wxPoint p[], wxSize s[], int count);

/**
 * @brief Calculates the resulting resolution of a image
 *
 * @param img                - the image (needed for info only)
 * @param max_bigger_side    - the maximal allowed bigger side after rescaling
 * @param x            [out] - the calculated x-size
 * @param y            [out] - the calculated y-size
 * @param rotated            - takes into account image rotation
 * @param original           - takes into account original image size
 *
 * If max(orig_x, orig_y) <= max_bigger_size, no calculations are performed
 * and the function returns false. Otherwise, x and y are scaled proportionally
 * and the return value is true.
*/
bool CalculateResizedSizes(const RawImg & img, int max_bigger_side, 
                           int &x, int &y, bool rotated = false, 
						   bool original = false);
						   
class SharedCounter {
	wxMutex lock;
	volatile int value;
public:
	void set(int value);
	void reset();
	int next();
	int decrement();
};


#endif // __TOOLS_H__
