#include <stdio.h>
#include <string.h>
#include <vector>
#include <string>
using namespace std;

/* Global vars section -> */

bool grayscale = false;
bool crossover = false;
bool swaplr = false;
int resize = 9999;
int jpegQuality = 85;

/* <- Global vars section */

/* Configuration vars handling section -> */

enum VarType {
	VAR_BOOL,
	VAR_INT,
	VAR_FLOAT,
	VAR_STRING
};

struct ConfigVar {
	char name[32];
	char desc[220];
	VarType type;
	union {
		bool  * as_bool;
		int   * as_int;
		float * as_float;
		string* as_string;
	} value;
	
	void ptrinit(bool * x) { value.as_bool = x; type = VAR_BOOL; }
	void ptrinit(int * x) { value.as_int = x; type = VAR_INT; }
	void ptrinit(float * x) { value.as_float = x; type = VAR_FLOAT; }
	void ptrinit(string * x) { value.as_string = x; type = VAR_STRING; }
};

static vector<ConfigVar> vars;

template <typename T>
void add_var(T* varptr, const char * varname, const char * vardesc)
{
	ConfigVar cvar;
	strcpy(cvar.name, varname);
	strcpy(cvar.desc, vardesc);
	cvar.ptrinit(varptr);
	vars.push_back(cvar);
}

void parser_init(void)
{
	add_var(&resize, "resize",
		"sets the desired output image size (applies to the larger side)");
	
	add_var(&jpegQuality, "jpegQuality",
		"output JPEG quality, generally useful: 5-95");
	
	add_var(&crossover, "crossover",
		"if present, create a stereo image of every two consecutive\n"
	   "\t\timages, e.g. 1.jpg, 2.jpg, 3.jpg will create a blend of 1&2 and\n"
	   "\t\ta blend of 2&3");
		
	add_var(&swaplr, "swaplr",
		"if present, left and right images will be swapped");
		
	add_var(&grayscale, "grayscale",
		"discard color data for anaglyphs");
}

/* <- Configuration vars handling section */

/* Actual parsing section -> */

static void usage(void)
{
	printf("asg - utility to process pictures into 3-D anaglyphs or cross-eyed ones.\n");
	printf("      By default, asg will process all files within the current directory.\n");
	printf("\nUsage: asg [options]\n");
	printf("\t\tThe following options are available:\n");
	for (unsigned i = 0; i < vars.size(); i++) {
		string s = vars[i].name;
		s = "-" + s + " - ";
		while (s.length() < 16) s = " " + s;
		printf("%s%s\n", s.c_str(), vars[i].desc);
	}
}

static bool process_arg(string arg, string context)
{
	string orig_arg = arg;
	if (arg.length() == 0) {
		printf("%s: error: empty argument!\n", context.c_str());
		return false;
	}
	
	if (arg[0] != '-') {
		printf("%s: error: unrecognized option: %s!\n", context.c_str(), arg.c_str());
		return false;
	}
	
	arg = arg.substr(1);
	if (arg.length() > 0 && arg[0] == '-') arg = arg.substr(1);
	if (arg.length() == 0) {
		printf("%s: error: empty argument!\n", context.c_str());
		return false;
	}
	
	if (arg == "h" || arg == "help") {
		usage();
		return false;
	}
	
	unsigned i = 0;
	while (i < arg.length() && arg[i] != '=') i++;
	if (i >= arg.length()) {
		arg += "=1";
	}
	
	string value = arg.substr(i+1);
	string varname = arg.substr(0, i);
	
	unsigned k = 0;
	while (k < vars.size() && vars[k].name != varname) k++;
	if (k >= vars.size()) {
		printf("%s: error: unrecognized option: %s!\n", context.c_str(), orig_arg.c_str());
		return false;
	}
	
	ConfigVar& v = vars[k];
	int resulto = 0;
	switch (v.type) {
		case VAR_INT:
		case VAR_BOOL:
		{
			int t;
			resulto = sscanf(value.c_str(), "%d", &t);
			if (resulto) {
				if (v.type == VAR_INT)
					*v.value.as_int = t;
				else
					*v.value.as_bool = t != 0;
			}
			break;
		}
		case VAR_FLOAT:
			resulto = sscanf(value.c_str(), "%f", v.value.as_float);
			break;
		case VAR_STRING:
			resulto = 1;
			*v.value.as_string = value;
			break;
	}
	if (resulto != 1) {
		printf("%s: error: unrecognized option: %s!\n", context.c_str(), orig_arg.c_str());
		return false;
	}
	
	return true;
}

bool parse_config(string fn)
{
	FILE * f = fopen(fn.c_str(), "rt");
	if (!f) return true;
	char line[200];
	bool ok = true;
	while (fgets(line, 200, f)) {
		int m = (int) strlen(line);
		if (m && line[m-1] == '\n')
			line[m-1] = 0;
		ok = ok && process_arg(line, fn);
	}
	fclose(f);
	return ok;
}

bool parse_cmdline(int argc, char **argv)
{
	bool ok = true;
	for (int i = 1; i < argc; i++)
		ok = ok && process_arg(argv[i], "command line");
	return ok;
}


/* <- Actual parsing section */
