/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <wx/wx.h>
#include <wx/image.h>
#ifdef HAVE_STDLIB_H
#undef HAVE_STDLIB_H
#endif
#define XMD_H
#include <cdjpeg.h>
#undef XMD_H
#include <math.h>

#include "bitmap.h"
#include "defaults.h"
#include "tools.h"
#include "resizer.h"
#include "preferences.h"

#ifdef _WIN32 // for alloca
#include <malloc.h>
#endif

typedef unsigned char * LPBYTE;

bool Trapped = false;
const char* CFile;

static void JpegTrap(j_common_ptr p)
{
	if (Trapped) return;
	Trapped = true;
	wxMessageBox(
		wxString::Format(wxT("Error reading JPEG file (%s)!"), CFile), 
		wxT("Error"), wxICON_ERROR);
	exit(1);
}

static inline unsigned clamp(float x)
{
	if (x < 0.0f) x = 0.0f;
	if (x > 1.0f) x = 1.0f;
	return ((int) (x*255.0f));
}

unsigned Rgb:: toRGB32() const
{
	return clamp(b) + ((clamp(g))<<8) + ((clamp(r))<<16);
}

void Rgb:: CreateFrom(unsigned x)
{
	b = ((x)       & 0xff) / 255.0f;
	g = ((x >>  8) & 0xff) / 255.0f;
	r = ((x >> 16) & 0xff) / 255.0f;
	a = ((x >> 24) & 0xff) / 255.0f;
}

RawImg:: RawImg()
{
	x = y = rotation = 0;
	data = NULL;
	FileName = wxEmptyString;
	exif = NULL;
	resolution = 0;
}

RawImg:: ~RawImg()
{
	if (data) Destroy();
	if (exif) {
		delete exif;
		exif = NULL;
	}
}

void RawImg:: Destroy()
{
	sse_free(data);
	data = NULL;
}

bool RawImg:: LoadFromJPG(const char * filename)
{
	struct jpeg_decompress_struct cinfo;
	struct jpeg_error_mgr jerr;
//	djpeg_dest_ptr dest_mgr = NULL;
	FILE * input_file;
	JDIMENSION num_scanlines;

	CFile = filename;

//   Initialize the JPEG decompression object with default error handling.
	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_decompress(&cinfo);

// Add some application-specific error messages (from cderror.h) 
	jerr.addon_message_table = NULL;//cdjpeg_message_table;
	jerr.first_addon_message = JMSG_FIRSTADDONCODE;
	jerr.last_addon_message = JMSG_LASTADDONCODE;
	jerr.error_exit = JpegTrap;

// Insert custom marker processor for COM and APP12.
// APP12 is used by some digital camera makers for textual info,
// so we provide the ability to display it as text.
// If you like, additional APPn marker types can be selected for display,
// but don't try to override APP0 or APP14 this way (see libjpeg.doc).
//

//  jpeg_set_marker_processor(&cinfo, JPEG_COM, print_text_marker);
//  jpeg_set_marker_processor(&cinfo, JPEG_APP0+12, print_text_marker);

// Open the input file. 
	if( (input_file = fopen( filename, "rb")) == NULL){
		return false;
	}

// Specify data source for decompression 
	jpeg_stdio_src( &cinfo, input_file);

// Read file header, set default decompression parameters
	(void) jpeg_read_header(&cinfo, TRUE);

// Adjust default decompression parameters by re-parsing the options 
//  file_index = parse_switches(&cinfo, argc, argv, 0, TRUE);

// dest_mgr->output_file = output_file;

// Start decompressor 
	(void) jpeg_start_decompress(&cinfo);

	data = (Rgb*) sse_malloc(cinfo.output_width*cinfo.output_height * sizeof(Rgb));

	int row_stride = cinfo.output_width * 4;
	LPBYTE *buffer = (*cinfo.mem->alloc_sarray)
		((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);

// Process data 
	while (cinfo.output_scanline < cinfo.output_height) {
    		num_scanlines = jpeg_read_scanlines(&cinfo, buffer,1);

		LPBYTE line=buffer[0];
		for( int i=0; i<(int)cinfo.output_width; i++) {
			Rgb pixel;
			if (cinfo.out_color_space==JCS_RGB) {
				int idx = i*3;
				pixel.r = float( line[idx+0])/255.0f;
				pixel.g = float( line[idx+1])/255.0f;
				pixel.b = float( line[idx+2])/255.0f;
				pixel.a = 1.0f;
			} else if (cinfo.jpeg_color_space==JCS_GRAYSCALE) {
				int idx = i;
				pixel.r = pixel.g = pixel.b = 
					float( line[idx])/255.0f;
				pixel.a = 1.0f;
			}
			int residx = ((cinfo.output_scanline-1)*cinfo.output_width+i);
			data[residx] = pixel;
		}
  	}

// Finish decompression and release memory.
// I must do it in this order because output module has allocated memory
// of lifespan JPOOL_IMAGE; it needs to finish before releasing memory.
//
	(void) jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);

// Close files, if we opened them
	if (input_file != stdin)
  		fclose(input_file);

	original_x = x = cinfo.image_width;
	original_y = y = cinfo.image_height;
	
	exif = new ExifWrapper;
	if (!exif->try_load(filename)) {
		delete exif;
		exif = NULL;
	}

	return true;
}

bool RawImg:: LoadFromBMP(const char *fn)
{
	FILE *f;
	int i, j, p, k;
	BMHEADER hd;
	BMiHEADER hi;
	int pal[256];
	int toread = 0;
	unsigned char *xx;
	int rowsz;
	unsigned short sign;
	
	if ((f = fopen(fn, "rb"))==NULL) {
		//printf("LoadBmp:Can't open file: `%s'\n", fn);
		return false;
	}
	if (!fread(&sign, 2, 1, f)) return false;
	if (sign!=itohs(19778)) {
		//printf("LoadBmp:`%s' is not a BMP file.\n", fn); 
		return false; 
	}
	if (!fread(&hd, sizeof(hd), 1, f)) return false;
	if (!fread(&hi, sizeof(hi), 1, f)) return false;
	
	/* header correctness checks */
	if (!(itohs(hi.bitsperpixel)==8 || itohs(hi.bitsperpixel)==24 || itohs(hi.bitsperpixel)==32)){
		//printf("LoadBmp:Cannot handle file format at %d bpp.\n",itohs(hi.bitsperpixel)); 
		return false;
	}
	if (itohs(hi.channels)!=1) {
		//printf("LoadBmp: cannot load multichannel .bmp!\n");
		return false;
	}
	/* ****** header is OK *******/
	
	// if image is 8 bits per pixel or less (indexed mode), read some palette data
	if (itohs(hi.bitsperpixel) <= 8) {
		toread = (1<<itohs(hi.bitsperpixel));
		if (itohl(hi.colors)) toread=itohl(hi.colors);
		if (!fread(pal, toread, 4, f)) return false;
	}
	toread = itohl(hd.bfImgOffset) - (54 + toread*4);
	if (toread)
	// skip the rest of the header
		for (i=0;i<toread;i++)
			fgetc(f);
	k = itohs(hi.bitsperpixel) / 8;
	rowsz = itohl(hi.x) * k;
	if (rowsz%4)
		rowsz = (rowsz / 4 + 1) * 4; // round the row size to the next exact multiple of 4
	xx = (unsigned char *) malloc(rowsz);
	p = itohl(hi.x) * itohl(hi.y);
	x = itohl(hi.x);
	y = itohl(hi.y);
	original_x = x;
	original_y = y;
	
	data = (Rgb *) sse_malloc(itohl(hi.x) * itohl(hi.y) * sizeof(Rgb));
	if (!data) {
		//printf("LoadBmp: cannot allocate memory for bitmap! Check file integrity!\n");
		fflush(stdout);
		free(xx);
		fclose(f);
		return false;
	}
	for (j=itohl(hi.y)-1;j>=0;j--) {// bitmaps are saved in inverted y
		if (!fread(xx, 1, rowsz, f)) {
			//printf("LoadBmp: short read while opening `%s', file is probably incomplete!\n", fn);
			free(xx);
			fclose(f);
			return 0;
		}
		p -= itohl(hi.x);
		for (i=0;i<itohl(hi.x);i++){ // actually read the pixels
			if (itohs(hi.bitsperpixel)>8)
				data[p++].CreateFrom(
					xx[i*k] + 
					((int) xx[i*k+1])*256 + 
					((int) xx[i*k+2])*65536 +
					0xff000000
					);
				else
				data[p++].CreateFrom(0xff000000|pal[xx[i*k]]);
			}
			p -= itohl(hi.x);
	}
	fclose(f);
	free(xx);
	return true;
}

bool RawImg:: LoadFromImageFile(const char *fn)
{
	int l = strlen(fn);
	if (tolower(fn[l-1]) == 'g') //assume JPEG
		return LoadFromJPG(fn);
	else
		if (tolower(fn[l-1]) == 'p') // assume BMP
			return LoadFromBMP(fn);
		else
			return false;
}

bool RawImg:: Resize(int new_x, int new_y)
{
	if (!data) 
		return false;
	if (new_x > x) new_x = x;
	if (new_y > y) new_y = y;
		
	//
	
	Rgb *line = (Rgb*) alloca(16+(1+new_x) * sizeof(Rgb));
	#define fixup(x) reinterpret_cast<Rgb*>(16l+(reinterpret_cast<long>(x)&-16l))
	line = fixup(line);
	// resize in X direction:
	if (new_x < x) {
		for (int j = 0; j < y; j++) {
			ArrayResize(data + j * x, x, line, new_x);
			memcpy(data + j * x, line, new_x * sizeof(Rgb));
		}
	}
	// resize in Y direction:
	if (new_y < y) {
		Rgb *temp = (Rgb*) sse_malloc(new_x * new_y * sizeof(Rgb));
		if (!temp)
			return false;
		Rgb *src_column = (Rgb*) alloca(16 + y * sizeof (Rgb) );
		src_column = fixup(src_column);
		Rgb *dst_column = (Rgb*) alloca(16 + (1+new_y) * sizeof(Rgb) );
		dst_column = fixup(dst_column);
		for (int i = 0; i < new_x; i++) {
			for (int j = 0; j < y; j++)
				src_column[j] = data[i+j*x];
			ArrayResize(src_column, y, dst_column, new_y);
			for (int j = 0; j < new_y; j++)
				temp[i + j*new_x] = dst_column[j];
		}
		sse_free(data);
		data = temp;
	}
	x = new_x;
	y = new_y;
	return true;
	#undef fixup
}


bool RawImg:: SaveToJPG(const char *fn, int quality, bool CreateFrame, 
						unsigned FrameColor, float opacity, float gamma,
                        ExifHandling exif_handling)
{
	int width, height;
	get_rotated_sizes(width, height);
	JSAMPLE* my_data = (JSAMPLE *) 
		CreateRGBPreview(CreateFrame, FrameColor, opacity, gamma);
	if (!my_data) return false;
	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;

	FILE * outfile;
	JSAMPROW row_pointer[1];
	int row_stride;
	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_compress(&cinfo);
	if ((outfile = fopen(fn, "wb")) == NULL) {
		free(my_data);
		return false;
	}
	jpeg_stdio_dest(&cinfo, outfile);
	cinfo.image_width = width;
	cinfo.image_height = height;
	cinfo.input_components = 3;
	cinfo.in_color_space = JCS_RGB;
	
	jpeg_set_defaults(&cinfo);
	jpeg_set_quality(&cinfo, quality, TRUE);
					/* limit to baseline-JPEG values */
	
	jpeg_start_compress(&cinfo, TRUE);
	row_stride = 3 * width;
	
	while (cinfo.next_scanline < cinfo.image_height) {
		row_pointer[0] = & my_data[cinfo.next_scanline * row_stride];
		(void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
	}
	jpeg_finish_compress(&cinfo);
	fclose(outfile);

	jpeg_destroy_compress(&cinfo);
	free(my_data);
	
	if (exif && preferences.preserve_exif && exif_handling != DONT_RETAIN_EXIF)
		exif->try_save(fn);
	
	return true;	
}

bool RawImg:: SaveToPNG(const char *fn, bool CreateFrame, float opacity, 
						float gamma)
{
	wxImage img;
	int width, height;
	get_rotated_sizes(width, height);

	img.Create(width, height);
	img.SetData(CreateRGBPreview(false, 0, 1.0f, gamma));
	if (CreateFrame) {
		img.InitAlpha();
		if (img.HasAlpha())
			img.SetAlpha(CreateAlphaPreview(opacity));
	}
	return img.SaveFile(fn, wxBITMAP_TYPE_PNG);
}

bool RawImg:: SaveToBMP(const char *fn)
{
	FILE *f;
	int i,j, rowsz;
	unsigned short msh=19778;
	BMHEADER hd;
	BMiHEADER hi;
	//char xx[RES_MAXX*3];
	char *xx = (char*) alloca(x*3+4);
	memset(xx, 0, x*3+4);
	
	
	if ((f=fopen(fn, "wb"))==NULL) {
		//printf("SaveBmp:can't open `%s' for writing, check permissions!\n", fn);
		return false;
	}
	// fill in the header:
	rowsz = x * 3;
	if (rowsz % 4)
		rowsz += 4 - (rowsz % 4); // each row in of the image should be filled with zeroes to the next multiple-of-four boundary
	hd.fs = rowsz*y + 54; //std image size :)
	hd.lzero = 0;
	hd.bfImgOffset = 54;
	hi.ihdrsize = 40;
	hi.x = x; hi.y = y;
	hi.channels = 1;
	hi.bitsperpixel = 24; //RGB format
	// set the rest of the header to default values:
	hi.compression = hi.biSizeImage = 0;
	hi.pixPerMeterX = hi.pixPerMeterY = 0;
	hi.colors = hi.colorsImportant = 0;
	
	fwrite(&msh, 2, 1, f); // write 'BM'
	fwrite(&hd, sizeof(hd), 1, f); // write file header
	fwrite(&hi, sizeof(hi), 1, f); // write image header
	
	for (j=y-1;j>=0;j--) {
		for (i=0;i<x;i++) {
			unsigned t = data[j*x+i].toRGB32();
			xx[i*3  ] = (0xff     & t);
			xx[i*3+1] = (0xff00   & t) >> 8;
			xx[i*3+2] = (0xff0000 & t) >> 16;
		}
		fwrite(xx, rowsz, 1, f);
	}
	fclose(f);
	return true;
}

bool RawImg:: SaveToImageFile(const char *fn, int quality)
{
	wxString g(fn);
	if (ExtensionIs(g, "bmp"))
		return SaveToBMP(fn);
	if (ExtensionIs(g, "png"))
		return SaveToPNG(fn);
	if (ExtensionIs(g, "jpg"))
		return SaveToJPG(fn, quality);
	return false;
}

bool RawImg:: AutoLoad()
{
	if (ExtensionIs(FileName, "jpg")) {
		bool res = LoadFromJPG(FileName.c_str());
		if (this->rotation == -1) {
			int orientation = 0;
			if (exif) {
				orientation = exif->get_rotation();
				if (-1 == orientation)
					orientation = 0;
			}
			this->rotation = orientation;
		}
		return res;
	} else {
		if (ExtensionIs(FileName, "bmp")) {
			return LoadFromBMP(FileName.c_str());
		} else {
			return false;
		}
	}
}


int RawImg:: get_x() const { return x; }
int RawImg:: get_y() const { return y; }
int RawImg:: get_original_x() const { return original_x; }
int RawImg:: get_original_y() const { return original_y; }
int RawImg:: get_size() const { return x*y; }
int RawImg:: get_rotation() const { return rotation; }
Rgb* RawImg:: get_data() const{ return data; }
void RawImg:: get_rotated_sizes(int & xsize, int & ysize) const
{
	if (rotation % 2) {
		xsize = y;
		ysize = x;
	} else {
		xsize = x;
		ysize = y;
	}
}

void RawImg:: get_original_rotated_sizes(int & xsize, int & ysize) const
{
	if (rotation % 2) {
		xsize = original_y;
		ysize = original_x;
	} else {
		xsize = original_x;
		ysize = original_y;
	}
}


void RawImg:: set_rotation(int new_rotation)
{
	rotation = new_rotation % 4;
}

void RawImg:: set_resolution(int new_res)
{
	resolution = new_res;
}

int RawImg:: get_resolution() const
{
	return resolution;
}

ExifWrapper* RawImg:: get_exif() const
{
	return exif;
}

int RawImg:: get_translated_index(int i, int j) const
{
	switch (rotation) {
		case 0: return i + j * x;
		case 1: return i * x + (x-1-j);
		case 2: return (x-1-i) + (y-1-j) * x;
		case 3: return (y-1-i)*x + j;
		default: return 0;
	}
}

static inline void U(float & opacity, float test)
{
	if (test < 1.0f)
		opacity *= test;
}

unsigned char* RawImg:: CreateRGBPreview(bool CreateFrame, 
					unsigned BackgroundColor, float opacity_in, float gamma)
{
	unsigned char* out = (unsigned char*) malloc(x*y*3);
	if (!out) return out;
		
	float back_r=1.0f, back_g=1.0f, back_b=1.0f;
	if (CreateFrame) {
		back_b = ((BackgroundColor     ) & 0xff) / 255.0f;
		back_g = ((BackgroundColor >> 8) & 0xff) / 255.0f;
		back_r = ((BackgroundColor >>16) & 0xff) / 255.0f;
	}
	int prevx, prevy;
	get_rotated_sizes(prevx, prevy);
	int idx = 0;
	for (int j = 0; j < prevy; j++)
		for (int i = 0; i < prevx; i++,idx++) {
			int srcidx = get_translated_index(i, j);
			if (srcidx >= x*y) srcidx = x*y-1;
			if (srcidx < 0) srcidx = 0;
			if (CreateFrame || opacity_in < 1.0f) {
				float opacity = opacity_in;
				if (CreateFrame) {
					U(opacity, (float) i / PREVIEW_FRAME);
					U(opacity, (float) (prevx-1-i) / PREVIEW_FRAME);
					U(opacity, (float) j / PREVIEW_FRAME);
					U(opacity, (float) (prevy-1-j) / PREVIEW_FRAME);
				}
				if (opacity < 1.0f) {
					opacity *= opacity;
					out[idx * 3    ] = 
						clamp(data[srcidx].r * gamma * opacity +
						      back_r * (1.0f- opacity));
					out[idx * 3 + 1] = 
						clamp(data[srcidx].g * gamma * opacity +
						      back_g * (1.0f- opacity));
					out[idx * 3 + 2] = 
						clamp(data[srcidx].b * gamma * opacity +
						      back_b * (1.0f- opacity));
								
					continue;
				}
			}
			out[idx * 3    ] = clamp(data[srcidx].r * gamma);
			out[idx * 3 + 1] = clamp(data[srcidx].g * gamma);
			out[idx * 3 + 2] = clamp(data[srcidx].b * gamma);
		}
	return out;
}

unsigned char* RawImg:: CreateAlphaPreview(float opacity_in)
{
	unsigned char* out = (unsigned char*) malloc(x*y);
	if (!out) return out;
		
	int prevx, prevy;
	get_rotated_sizes(prevx, prevy);
	int idx = 0;
	for (int j = 0; j < prevy; j++)
		for (int i = 0; i < prevx; i++,idx++) {
			int srcidx = get_translated_index(i, j);
			if (srcidx >= x*y) srcidx = x*y-1;
			if (srcidx < 0) srcidx = 0;
			float opacity = opacity_in;
			U(opacity, (float) i / PREVIEW_FRAME);
			U(opacity, (float) (prevx-1-i) / PREVIEW_FRAME);
			U(opacity, (float) j / PREVIEW_FRAME);
			U(opacity, (float) (prevy-1-j) / PREVIEW_FRAME);
			opacity *= opacity;
			if (opacity > 1.0f) opacity = 1.0f;
			out[idx] = (int) (255.0f * opacity);
		}
	return out;
}

bool RawImg:: ExtractExifDateTime()
{
	ExifWrapper exif;
	if (!exif.try_load(FileName.c_str())) return false;
	if (!exif.get_datetime(exif_datetime)) return false;
	return true;
}

time_t RawImg:: ExifDateTime() const
{
	return exif_datetime;
}

bool RawImg:: GetFullExifInfo(FullExifData  & data)
{
	if (!exif) return false;
	return exif->get_full_exif(data);
}
