/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <wx/wx.h>
#include <wx/spinctrl.h>
#include <wx/colordlg.h>

#include "project.h"
#include "configuration.h"
#include "tools.h"
#include "main.h"
#include "customfontdlg.h"
#include "preferences.h"

wxString ThumbFmtChoices[] = { wxT("JPG"), wxT("PNG") };
wxString FileInfoChoices[] = { wxT("Nothing"), wxT("Filename"), wxT("Filename + Filesize"),
		wxT("Filename + Dimensions"), wxT("Everything")};
wxString RollOverChoices[] = { wxT("None"), wxT("Highlight"), wxT("Change opacity"), wxT("Both") };
wxString GalleryTypeChoices[] = { wxT("Page With Thumbs"), wxT("One Picture Per Page") };
wxString FontChoices[] = {
	wxT(DEFAULT_FONT_CHOICE),
	wxT("Courier, Courier New, Monospace, Andale Mono"),
	wxT("Verdana, Arial, Helvetica, sans-serif"),
	wxT("Times New Roman, Georgia, Serif"),
};
unsigned FileInfoMasks[] = {
	0,
	PICINFO_FILENAME,
	PICINFO_FILENAME|PICINFO_FILESIZE,
	PICINFO_FILENAME|PICINFO_DIMENSIONS,
	PICINFO_FILENAME|PICINFO_FILESIZE|PICINFO_DIMENSIONS
};

enum {
	Conf_Text1=501,
	Conf_Text2,
	Conf_iPath_Browse,
	Conf_oPath_Browse,
	Conf_Spin_PicSize,
	Conf_Spin_ThumbSize,
	Conf_Combo_ThumbFormat,
	Conf_Slider_JPGQ,
	Conf_Button_FgColor,
	Conf_Button_BgColor,
	Conf_CheckBox_BgImage,
	Conf_Spin_PicsPerRow,
	Conf_Spin_PicsPerGallery,
	Conf_Combo_InfoPerFile,
	Conf_CheckBox_MultiPage,
	Conf_Combo_Rollover,
	Conf_Combo_Font,
};

BEGIN_EVENT_TABLE(ConfigurationDialog, wxDialog)
	EVT_BUTTON(wxID_OK, ConfigurationDialog:: OnOK)
	EVT_BUTTON(wxID_CANCEL, ConfigurationDialog:: OnCancel)
	EVT_TEXT(Conf_Text1, ConfigurationDialog:: OnPathsChange)
	EVT_TEXT(Conf_Text2, ConfigurationDialog:: OnPathsChange)
	EVT_BUTTON(Conf_iPath_Browse, ConfigurationDialog:: OnInputPathBrowse)
	EVT_BUTTON(Conf_oPath_Browse, ConfigurationDialog:: OnOutputPathBrowse)
	EVT_BUTTON(Conf_Button_FgColor, ConfigurationDialog:: OnFGColorClick)
	EVT_BUTTON(Conf_Button_BgColor, ConfigurationDialog:: OnBGColorClick)
	EVT_CHECKBOX(Conf_CheckBox_MultiPage, ConfigurationDialog:: OnMultiPageClick)
	EVT_CHECKBOX(Conf_CheckBox_BgImage, ConfigurationDialog:: OnBGImageClick)
	EVT_COMMAND_SCROLL(Conf_Slider_JPGQ, ConfigurationDialog:: OnSliderChange)
	EVT_COMBOBOX(Conf_Combo_Font, ConfigurationDialog:: OnFontChange)
END_EVENT_TABLE()


ConfigurationDialog:: ConfigurationDialog(bool isNew) 
		: wxDialog(globMainFrame, 
		wxID_ANY, 
		isNew?wxT("New Project Settings"):wxT("Project Configuration"),
		wxDefaultPosition,
		wxSize(600, 504)
		)
{
	/* Create the OK and Cancel buttons */
	button_OK = new wxButton(this, wxID_OK);
	wxButton *button_Cancel = new wxButton(this, wxID_CANCEL);
	button_OK->SetSize(180, 444, -1, -1);
	button_Cancel->SetSize(300, 444, -1, -1);
	button_OK->Disable();

	/* Create Title selection box and text */
	wxStaticText *qq = new wxStaticText(this, -1,
		wxT("Gallery Title:"), wxPoint(20, 24));
	TitleBox = new wxTextCtrl(this, -1, wxT(""), wxPoint(EndX(qq)+10, 20), 
		wxSize(320, 24));
	qq->Refresh();
	TitleBox->SetValue(ProjInfo.Title);
	
	/* Create the Gallery Type selector */
	/*
	wxStaticText *gtgt = new wxStaticText(this, -1, wxT("Gallery Type:"),
		wxPoint(EndX(TitleBox) + 10, 24));
	galleryType = new wxComboBox(this, -1, GalleryTypeChoices[0], 
		wxPoint(EndX(gtgt) + 5, 20), wxSize(170, 24), 2,
		GalleryTypeChoices, wxCB_READONLY);
	galleryType->SetValue(GalleryTypeChoices[(int) ProjInfo.galleryType]);
	*/
	
	/* Create the path selection stuff */
	wxStaticText *tt1 = new wxStaticText(this, -1, 
		wxT("Image Collection Directory:"), wxPoint(20, 64));
	wxStaticText *tt2 = new wxStaticText(this, -1, 
		wxT("Gallery Output Directory:"), wxPoint(20, 104));
	int XStart = 10 + IMax(EndX(tt1), EndX(tt2));
	wxRect wndRect = GetRect();
	
	int XSize = wndRect.width - 90 - XStart;
	InputPath = new wxTextCtrl(this, Conf_Text1, "",
		wxPoint(XStart, 60), wxSize(XSize, 24));
	OutputPath = new wxTextCtrl(this, Conf_Text2, "",
		wxPoint(XStart, 100), wxSize(XSize, 24));
	InputPath->SetValue(ProjInfo.iPath);
	OutputPath->SetValue(ProjInfo.oPath);
	wxButton *buttonInputPath = new wxButton(this, Conf_iPath_Browse, 
		"Browse...", wxPoint(XStart + XSize + 10, 60), wxSize(72, 24));
	wxButton *buttonOutputPath= new wxButton(this, Conf_oPath_Browse, 
		"Browse...", wxPoint(XStart + XSize + 10, 100), wxSize(72, 24));
	/*
	wxRect gtrect = galleryType->GetRect();
	galleryType->SetSize(EndX(buttonOutputPath) - gtrect.x, -1);
	*/
	
	/* Create the Spin Controls for the picture and thumbnail size */		
	PicSZ = new wxSpinCtrl(this, Conf_Spin_PicSize);
	ThumbSZ = new wxSpinCtrl(this, Conf_Spin_ThumbSize);
	PicSZ->SetRange(4, 10000);
	ThumbSZ->SetRange(4, 600);
	PicSZ->SetValue(ProjInfo.PicSize);
	ThumbSZ->SetValue(ProjInfo.ThumbSize);
	wxStaticText *tt3 = new wxStaticText(this, -1,
		wxT("Resize pictures to: "), wxPoint(20, 140));
	PicSZ->SetSize(EndX(tt3) + 4,  136, -1, -1);
	wxStaticText *tt4 = new wxStaticText(this, -1, 
		wxT("Size thumbnails at: "), wxPoint(20, 180));
	ThumbSZ->SetSize(EndX(tt4) + 4, 176, -1, -1);
	
	wxStaticText *ee1 = new wxStaticText(this, -1,
		wxT("Pixels (at the larger side)"), wxPoint(EndX(PicSZ)+4, 140));
	wxStaticText *ee2 = new wxStaticText(this, -1,
		wxT("Pixels (at the larger side)"), wxPoint(EndX(ThumbSZ)+4, 180));
	ee1->Refresh(); ee2->Refresh();
	
	/* If not creating a new project, disable the input path selection */
	
	if (!isNew) {
		tt1->Disable();
		InputPath->Disable();
		buttonInputPath->Disable();
		tt2->Enable();
		OutputPath->Enable();
		buttonOutputPath->Enable();
	} else {
		tt1->Enable();
		InputPath->Enable();
		buttonInputPath->Enable();
	}
	
	/* Create the PNG/JPG thumbnail Combobox */
	wxStaticText *cb1 = new wxStaticText(this, -1,
		wxT("Thumbnail format: "), wxPoint(20, 220));
	cb1->Refresh();
	ThumbFormat = new wxComboBox(this, Conf_Combo_ThumbFormat,
		wxT("JPG"), wxPoint(EndX(cb1) + 8, 216), wxDefaultSize, 2, 
		ThumbFmtChoices, wxCB_READONLY);
	if (ProjInfo.ThumbFormat == FORMAT_PNG)
		ThumbFormat->SetValue(wxT("PNG"));
	
	/* Create a ComboBox for File info selection */
	wxStaticText *cb2 = new wxStaticText(this, -1,
		wxT("Show File Info: "), wxPoint(EndX(ThumbFormat)+20, 220));
	cb2->Refresh();
	FileInfo = new wxComboBox(this, Conf_Combo_InfoPerFile,
		wxT("Nothing"), wxPoint(EndX(cb2) + 8, 216), wxSize(180, -1), 5,
		FileInfoChoices, wxCB_READONLY);
	int i = 4;
	while (i>=0 && ProjInfo.InfoPerImg != FileInfoMasks[i]) i--;
	if (i>=0) {
		FileInfo->SetValue(FileInfoChoices[i]);
	}

	/* Create a JPEG Quality selector */
	jpeg1 = new wxStaticText(this, -1, wxString::Format(
		wxT("JPEG Quality: %3d"), ProjInfo.JpegQuality), wxPoint(20, 260)
		);
	JpegQualitySlider = new wxSlider(this, Conf_Slider_JPGQ, 
		ProjInfo.JpegQuality, 1, 100,
		wxPoint(EndX(jpeg1)+16, 256), wxSize(100, 24), 
		wxSL_HORIZONTAL);
	
	/* Create a button to change the Text foreground color */
	wxStaticText *col0 = new wxStaticText(this, -1,
		wxT("Text Color: "), wxPoint(EndX(JpegQualitySlider)+ 10, 260));
	col0->Refresh();
	fgcolorButton = new wxButton(this, Conf_Button_FgColor,
		wxString::Format(wxT("#%06x"), ProjInfo.TextColor),
		wxPoint(EndX(col0) + 5, 256));
	
	/* Create a combobox to select HTML page font: */
	wxStaticText *fnt0 = new wxStaticText(this, -1, wxT("Font: "),
		wxPoint(EndX(fgcolorButton)+8, 260));
	font = new wxComboBox(this, Conf_Combo_Font, ProjInfo.font, 
		wxPoint(EndX(fnt0), 256), wxSize(130,-1), 
		sizeof(FontChoices)/sizeof(FontChoices[0]), FontChoices, wxCB_DROPDOWN);
	for (int i = 0; i < 4; i++)
		if (preferences.custom_fonts[i][0])
			font->Append(preferences.custom_fonts[i]);
	font->Append(wxT("Custom..."));
	
	/* Create a button to change the Gallery background color */
	wxStaticText *col1 = new wxStaticText(this, -1,
		wxT("Gallery Background Color: "), wxPoint(20, 300));
	col1->Refresh();
	bgcolorButton = new wxButton(this, Conf_Button_BgColor, 
		wxString::Format(wxT("#%06x"), ProjInfo.BackgroundColor),
		wxPoint(EndX(col1) + 8, 296));

	/* Create a Combobox and text input field to enter a background image
	   for the gallery */
	bgImageCB = new wxCheckBox(this, Conf_CheckBox_BgImage,
		wxT("Background image"), wxPoint(EndX(bgcolorButton)+20, 300));
	bgImageText = new wxTextCtrl(this, -1, wxT(""), 
		wxPoint(EndX(bgImageCB)+4, 296), wxSize(120, 24));

	if (ProjInfo.BackgroundImage.empty()) {
		bgImageText->Disable();
		bgImageCB->SetValue(false);
	} else {
		bgImageText->Enable();
		bgImageText->SetValue(ProjInfo.BackgroundImage);
		bgImageCB->SetValue(true);
	}
	
	/* Create a Spin Control to select Pictures per Row */
	wxStaticText *ppr = new wxStaticText(this, -1,
		wxT("Images per Row: "), wxPoint(20, 340));
	ppr->Refresh();
	PicsPerRow = new wxSpinCtrl(this, Conf_Spin_PicsPerRow);
	PicsPerRow->SetSize(EndX(ppr)+8, 336, 40, -1);
	PicsPerRow->SetRange(1, 16);
	PicsPerRow->SetValue(ProjInfo.PicsPerRow);
	
	/* Create a Spin Control to select number of galleries and pictures per
	   Gallery page
	*/
	
	chkBox = new wxCheckBox(this, Conf_CheckBox_MultiPage, 
		wxT("Multi-page Gallery"), wxPoint(EndX(PicsPerRow)+20, 340));
	chkBox->SetValue(ProjInfo.PicsPerGallery!=0);
	PicsPerGallery = new wxSpinCtrl(this, Conf_Spin_PicsPerGallery);
	PicsPerGallery->SetSize(EndX(chkBox)+8, 336, 56, -1);
	PicsPerGallery->SetRange(1, 2000);
	PicsPerGallery->SetValue(ProjInfo.PicsPerGallery);
	if (!ProjInfo.PicsPerGallery)
		PicsPerGallery->Disable();
	else
		PicsPerGallery->Enable();
	
	wxStaticText *imgs = new wxStaticText(this, -1,
		wxT("images per page"), wxPoint(EndX(PicsPerGallery)+4, 340));
	imgs->Refresh();
	
	/* Create a "Rollover Type" Combobox */
	
	wxStaticText *roll = new wxStaticText(this, -1, 
		wxT("Rollover thumbnails type:"), wxPoint(20, 380));
	
	Rollover = new wxComboBox(this, Conf_Combo_Rollover,
		wxT("None"), wxPoint(EndX(roll) + 8, 376), wxSize(180, -1), 4,
		RollOverChoices, wxCB_READONLY);
	Rollover->SetValue(RollOverChoices[ProjInfo.Rollover]);
	
	/* Create a "Add Exif stats" Checkbox */
	exif_stats = new wxCheckBox(this, wxID_ANY, 
	    wxT("Add &Exif stats (if available)"), wxPoint(20, 404), wxDefaultSize);
	exif_stats->SetValue(ProjInfo.exif_stats);
	
	/* Set the focus to the OK button */

	button_OK->SetFocus();
}

void ConfigurationDialog:: OnOK(wxCommandEvent& event)
{
	ProjInfo.iPath = PathLastSlashStrip( InputPath->GetValue());
	ProjInfo.oPath = PathLastSlashStrip(OutputPath->GetValue());
	ProjInfo.Title = TitleBox->GetValue();
	if (ProjInfo.Title.IsEmpty())
		ProjInfo.Title = ProjInfo.iPath;
	ProjInfo.PicSize = PicSZ->GetValue();
	ProjInfo.ThumbSize = ThumbSZ->GetValue();
	ProjInfo.PicsPerRow= PicsPerRow->GetValue();
	if (chkBox->GetValue())
		ProjInfo.PicsPerGallery = PicsPerGallery->GetValue();
	else
		ProjInfo.PicsPerGallery = 0;
	if (wxT("PNG") == ThumbFormat->GetValue())
		ProjInfo.ThumbFormat = FORMAT_PNG;
	else
		ProjInfo.ThumbFormat = FORMAT_JPEG;

	wxString ss = FileInfo->GetValue();
	for (int i = 0; i < 5; i++)
		if (ss == FileInfoChoices[i]) {
			ProjInfo.InfoPerImg = FileInfoMasks[i];
			break;
		}
	ss = Rollover->GetValue();
	for (int i = 0; i < 4; i++) 
		if (ss == RollOverChoices[i]) {
			ProjInfo.Rollover = (RolloverState) i;
			break;
		}
	sscanf(bgcolorButton->GetLabel().c_str(), "#%x", &ProjInfo.BackgroundColor);
	sscanf(fgcolorButton->GetLabel().c_str(), "#%x", &ProjInfo.TextColor);

	ProjInfo.JpegQuality = JpegQualitySlider->GetValue();
	if (bgImageCB->GetValue()) {
		ProjInfo.BackgroundImage = bgImageText->GetValue();
	} else ProjInfo.BackgroundImage = "";
	/*
	if (galleryType->GetValue() == "Page With Thumbs")
		ProjInfo.galleryType = GT_PAGE_WITH_THUMBS;
	else if (galleryType->GetValue() == "One Picture Per Page")
		ProjInfo.galleryType = GT_ONE_PICTURE_PER_PAGE;
	*/
	ProjInfo.font = font->GetValue();
	ProjInfo.exif_stats = exif_stats->GetValue();
	EndModal(0);
}
void ConfigurationDialog:: OnCancel(wxCommandEvent& event)
{
	EndModal(-1);
}

void ConfigurationDialog:: OnPathsChange(wxCommandEvent& event)
{
	bool isOK = 
		path_is_ok(PathLastSlashStrip(InputPath->GetValue())) && 
		path_is_ok(PathLastSlashStrip(OutputPath->GetValue()));
	if (isOK)
		button_OK->Enable();
	else
		button_OK->Disable();
}

void ConfigurationDialog:: OnInputPathBrowse(wxCommandEvent& event)
{
	const wxString& dir = wxDirSelector(wxT("Choose a folder"),
		preferences.last_input_dir);
	if ( !dir.empty() ){
		InputPath->SetValue(dir);
		strcpy(preferences.last_input_dir, dir.c_str());
	}
}

void ConfigurationDialog:: OnOutputPathBrowse(wxCommandEvent& event)
{
	const wxString& dir = wxDirSelector(wxT("Choose a folder"),
		preferences.last_output_dir);
	if ( !dir.empty() ){
		OutputPath->SetValue(dir);
		strcpy(preferences.last_output_dir, dir.c_str());
	}
}

void ConfigurationDialog:: OnFGColorClick(wxCommandEvent& event)
{
	wxString ColString = fgcolorButton->GetLabel();
	wxColor iColor(GetHex(ColString, 1), GetHex(ColString, 3), 
			GetHex(ColString, 5));
	wxColor out = wxGetColourFromUser(this, iColor);
	if (out.Ok()) {
		ColString = wxString::Format(wxT("#%02x%02x%02x"), 
				out.Red(), out.Green(), out.Blue());
		fgcolorButton->SetLabel(ColString);
		ProjInfo.BackgroundColor = out.Blue() + (out.Green()<<8)
			+ (out.Red() << 16);
	}
}

void ConfigurationDialog:: OnBGColorClick(wxCommandEvent& event)
{
	wxString ColString = bgcolorButton->GetLabel();
	wxColor iColor(GetHex(ColString, 1), GetHex(ColString, 3), 
			GetHex(ColString, 5));
	wxColor out = wxGetColourFromUser(this, iColor);
	if (out.Ok()) {
		ColString = wxString::Format(wxT("#%02x%02x%02x"), 
				out.Red(), out.Green(), out.Blue());
		bgcolorButton->SetLabel(ColString);
		ProjInfo.BackgroundColor = out.Blue() + (out.Green()<<8)
			+ (out.Red() << 16);
	}
}

void ConfigurationDialog:: OnMultiPageClick(wxCommandEvent & event)
{
	if (chkBox->GetValue()) {
		PicsPerGallery->Enable();
	} else {
		PicsPerGallery->Disable();
	}
}

void ConfigurationDialog:: OnBGImageClick(wxCommandEvent& event)
{
	if (bgImageCB->GetValue()) {
		bgImageText->Enable();
		wxString fn = wxFileSelector(wxT("Choose Gallery background image"),
			InputPath->GetValue(),
			wxEmptyString, wxT("*.bmp;*.jpg;*.gif;*.png"), 
			wxT("All image files (*.bmp, *.jpg, *.gif, *.png)|*.bmp;*.jpg;*.gif;*.png|"
		        "JPEG Files (*.jpg)|*.jpg|"
				"Windows Bitmap Files (*.bmp)|*.bmp|"
				"GIF Files (*.gif)|*.gif|"
				"PNG Files (*.png)|*.png"), wxOPEN | wxFILE_MUST_EXIST, this);
		if (fn.empty()) {
			bgImageText->SetValue(wxT(""));
			bgImageText->SetFocus();
		} else {
			bgImageText->SetValue(fn);
		}
		
	} else {
		bgImageText->Disable();
	}
}

void ConfigurationDialog:: OnSliderChange(wxScrollEvent& event)
{
	jpeg1->SetLabel(wxString::Format(wxT("JPEG Quality: %3d"), 
		JpegQualitySlider->GetValue()));
}

void ConfigurationDialog:: OnFontChange(wxCommandEvent& event)
{
	if (wxT("Custom...") == font->GetValue()) {
		CustomFontDlg *cfd = new CustomFontDlg(this);
		if (cfd->ShowModal() == wxID_OK) {
			wxString result = cfd->GetResult();
			font->Delete(font->FindString(wxT("Custom...")));
			font->Append(result);
			font->SetValue(result);
			font->Append(wxT("Custom..."));
			strcpy(preferences.custom_fonts[preferences.custom_font_idx++],
				result.c_str());
			preferences.custom_font_idx %= 4;
		}
		cfd->Destroy();
	}
}
