/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "customfontdlg.h"
#include "defaults.h"
#include <wx/fontdlg.h>

enum {
	FontDlg_List=1001,
	FontDlg_Add,
	FontDlg_Rem,
	FontDlg_Up,
	FontDlg_Down,
};

BEGIN_EVENT_TABLE(CustomFontDlg, wxDialog)
	EVT_LISTBOX(FontDlg_List, CustomFontDlg:: OnListSelect)
	EVT_BUTTON(FontDlg_Add, CustomFontDlg:: OnAdd)
	EVT_BUTTON(FontDlg_Rem, CustomFontDlg:: OnRem)
	EVT_BUTTON(FontDlg_Up, CustomFontDlg:: OnUp)
	EVT_BUTTON(FontDlg_Down, CustomFontDlg:: OnDown)
END_EVENT_TABLE()

CustomFontDlg::CustomFontDlg(wxWindow *parent) : 
	wxDialog(parent, -1, wxT("Choose custom font"), wxDefaultPosition,
	wxSize(350, 320+WINDOWSVADD))
{
	/* Create OK and Cancel buttons */
	okb = new wxButton(this, wxID_OK, wxEmptyString,
		wxPoint( 75, 280), wxSize(90, 30));
	wxButton *cancelButton = new wxButton(this, wxID_CANCEL, wxEmptyString,
		wxPoint(185, 280), wxSize(90, 30));
	
	cancelButton->Refresh();
	
	/* Create hint text */
	wxStaticText *text1 = new wxStaticText(this, -1, 
		wxT("List of fonts to use for the HTML text\n")
		wxT("(the first available will be used by the browser)"),
		wxPoint(10, 10));
	text1->Refresh();
		
	/* Create font list box */
	fontlist = new wxListBox(this, FontDlg_List, wxPoint(10, 50), wxSize(220, 210));
	
	/* Create management buttons */
	wxSize bsize(100, 30);
	add = new wxButton(this, FontDlg_Add, "&Add font", wxPoint(240,  50), bsize);
	rem = new wxButton(this, FontDlg_Rem, "&Delete",   wxPoint(240, 110), bsize);
	goup   = new wxButton(this, FontDlg_Up,   "&Up",   wxPoint(240, 170), bsize);
	godown = new wxButton(this, FontDlg_Down, "D&own", wxPoint(240, 230), bsize);	
	
	review();
}

void CustomFontDlg::review(void)
{
	int r = fontlist->GetSelection();
	int n = fontlist->GetCount();
	okb->Enable(n > 0);
	if (r == wxNOT_FOUND) {
		rem->Disable();
		goup->Disable();
		godown->Disable();
	} else {
		rem->Enable();
		goup->Enable(r != 0);
		godown->Enable(r != n-1);
	}
}

void CustomFontDlg::OnListSelect(wxCommandEvent&)
{
	review();
}

void CustomFontDlg::OnAdd(wxCommandEvent&)
{
	wxFont thefont = wxGetFontFromUser(this);
	if (thefont.Ok()) {
		fontlist->Append(thefont.GetFaceName());
	}
	review();
}

void CustomFontDlg::OnRem(wxCommandEvent&)
{
	fontlist->Delete(fontlist->GetSelection());
	review();
}

void CustomFontDlg::OnUp(wxCommandEvent&)
{
	int r = fontlist->GetSelection();
	if (r == wxNOT_FOUND || r == 0) return;
	wxString a = fontlist->GetString(r-1);
	wxString b = fontlist->GetString(r);
	fontlist->SetString(r-1, b);
	fontlist->SetString(r, a);
	fontlist->SetSelection(r-1);
	review();
}

void CustomFontDlg::OnDown(wxCommandEvent&)
{
	int r = fontlist->GetSelection();
	if (r == wxNOT_FOUND || r == (int) fontlist->GetCount()-1) return;
	wxString a = fontlist->GetString(r+1);
	wxString b = fontlist->GetString(r);
	fontlist->SetString(r+1, b);
	fontlist->SetString(r, a);
	fontlist->SetSelection(r+1);
	review();
}

wxString CustomFontDlg::GetResult(void)
{
	wxString result = wxEmptyString;
	
	for (int i = 0; i < (int) fontlist->GetCount(); i++)	{
		if (i) result += wxT(", ");
		result += fontlist->GetString(i);
	}
	return result;
}
