/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <wx/wx.h>
#include <vector>
#include "defaults.h"
#include "project.h"
#include "tools.h"
using std::vector;



wxString GetIndexName(int idx)
{
	if (idx)
		return wxString::Format("index%d.html", idx);
	else
		return "index.html";
}

void Project:: PutGallerySelector(FILE *f, int idx, int all)
{
	fprintf(f, "<p><font face=\"Arial, Helvetica, sans-serif\">Page: [ ");
	if (0 != idx) {
		fprintf(f, "<a href=\"%s\">Prev</a> ", GetIndexName(idx-1).c_str());
	} else {
		fprintf(f, "Prev ");
	}
	for (int i = 0; i < all; i++)
		if (i == idx) {
			fprintf(f, "<b>%d</b> ", i+1);
		} else {
			fprintf(f, "<a href=\"%s\">%d</a> ", GetIndexName(i).c_str(), i+1);
		}
	if (all - 1 != idx) {
		fprintf(f, "<a href=\"%s\">Next</a> ", GetIndexName(idx+1).c_str());
	} else {
		fprintf(f, "Next ");
	}

	fprintf(f, "]</font></p>\n");
}

bool Project:: CreateHtmlPage(int pgIdx, int pgCount)
{
	FILE *f;
	wxString htmlFile = SlashExtend(pi.oPath) + GetIndexName(pgIdx);
	f = fopen(htmlFile.c_str(), "wt");
	if (!f) return false;
	int ImgPerRow = pi.PicsPerRow;

	fprintf(f, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n");
	fprintf(f, "\"http://www.w3.org/TR/html4/loose.dtd\">\n");
	fprintf(f, "<html>\n<head>\n");
	fprintf(f, "<title>Gallery for %s</title>\n", pi.Title.c_str());
	fprintf(f, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");
	if (pi.Rollover)
		WriteJavaScript(f);
	fprintf(f, "</head>\n\n");

	unsigned brack_col = Interpolate(pi.BackgroundColor, 0.2f, pi.TextColor);

	fprintf(f, "<body bgcolor=\"#%06X\" ", pi.BackgroundColor);
	if (!pi.BackgroundImage.empty()) {
		wxString bg_filename = GetFn(pi.BackgroundImage);
		wxString destfn = SlashExtend(pi.oPath) + bg_filename;
		myCopyFile(pi.BackgroundImage.c_str(), destfn.c_str());
		fprintf(f, "background=\"%s\" ", bg_filename.c_str());
	}
	fprintf(f, "text=\"#%06X\" ", pi.TextColor);
	if (pi.Rollover) {
		fprintf(f, "onLoad=\"MM_preloadImages(");
		bool first = true;
		int kount = GetCount();
		ProjectIterator startImg = begin();
		if (pi.PicsPerGallery) {
			kount = pi.PicsPerGallery;
			if (pgIdx == pgCount - 1) {
				kount = GetCount() - pgIdx * pi.PicsPerGallery;
			}
			startImg += pi.PicsPerGallery * pgIdx;
		}
		ProjectIterator i = startImg;
		for (int ii = 0; ii < kount; ii++, i++) {
			wxString fn = (*i)->outFileName;
			fprintf(f, "%s'thumbs/r_%s'", (first?"":","), MakeExt(fn).c_str());
			first = false;
		}
		fprintf(f, ")\" ");
	}
	fprintf(f, ">\n");
	fprintf(f, "<p>&nbsp;</p>\n");
	fprintf(f, "<h1 align=\"center\"><font face=\"%s\">", pi.font.c_str());
	fprintf(f, "Gallery for <font color=\"#%06x\">[</font><b>%s</b>", brack_col, pi.Title.c_str());
	fprintf(f, "<font color=\"#%06x\">]</font></font></h1>\n", brack_col);
	fprintf(f, "<h6 align=\"center\"><font face=\"%s\">", pi.font.c_str());
	fprintf(f, "[<i>Number of Images:</i> <b>%d</b>]</font></h6>\n", GetCount());
	fprintf(f, "<hr>\n<center>\n");
	if (pi.PicsPerGallery) {
		PutGallerySelector(f, pgIdx, pgCount);
	}
	fprintf(f, "<table>\n");

	int kount = GetCount();
	ProjectIterator startImg = begin();
	if (pi.PicsPerGallery) {
		kount = pi.PicsPerGallery;
		if (pgIdx == pgCount - 1) {
			kount = GetCount() - pgIdx * pi.PicsPerGallery;
		}
		startImg += pi.PicsPerGallery * pgIdx;
	}
	ProjectIterator i = startImg;
	for (int ii = 0; ii < kount; ii++, i++) {
		if (ii % ImgPerRow == 0)
			fprintf(f, "<tr>\n");
		fprintf(f, "<td align='center'>\n");
		wxString fn = ((*i)->outFileName);
		int x, y;
		(*i)->get_rotated_sizes(x, y);
		wxString roll1(""), roll2("");
		if (pi.Rollover) {
			roll1 = wxString::Format(" onMouseOut=\"MM_swapImgRestore()\" onMouseOver=\"MM_swapImage('Image%d','','thumbs/r_%s',1)\"", 
									ii+1, MakeExt(fn).c_str());
			roll2 = wxString::Format("name=\"Image%d\" ", ii+1);
		}
		fprintf(f, "<a href=\"%s\"%s><img %sborder=\"0\" src=\"thumbs/%s\" width=\"%d\" height=\"%d\" alt=\"thumbs/%s\"/></a>\n",
			MakeBMPH(fn).c_str(), roll1.c_str(), roll2.c_str(), 
			MakeExt(fn).c_str(), x, y, fn.c_str());
		if (pi.InfoPerImg) {
			fprintf(f, "<div><font size=\"-1\" face=\"%s\">", pi.font.c_str());
			bool first = true;
			if (pi.InfoPerImg & PICINFO_FILENAME) {
				fprintf(f, "%s", fn.c_str());
				first = false;
			}
			if (pi.InfoPerImg & PICINFO_FILESIZE) {
				if (!first) fprintf(f, " ");
				fprintf(f, "[<i>%dK</i>]", GetFileSizeInKB(SlashExtend(pi.oPath) + fn));
				first = false;
			}
			if (pi.InfoPerImg & PICINFO_DIMENSIONS) {
				if (!first) fprintf(f, "<br>");
				int ox, oy;
				(*i)->get_original_rotated_sizes(ox, oy);
				scale_sizes(ox, oy, pi.PicSize);
				fprintf(f, "<font size=\"-2\">%dx%d pixels</font>", ox, oy);
				first = false;
			}
			if (first) fprintf(f, "&nbsp;");
			fprintf(f, "</font></div>\n");
		}
		fprintf(f, "</td>\n");
		if (ii % ImgPerRow == ImgPerRow - 1)
			fprintf(f, "</tr>\n");

	}

	if ((kount-1) % ImgPerRow != ImgPerRow - 1)
		fprintf(f, "</tr>");
	fprintf(f, "</table>\n");
	if (pi.PicsPerGallery) {
		PutGallerySelector(f, pgIdx, pgCount);
	}
	fprintf(f, "</center>\n");
	fprintf(f, "<p>&nbsp;</p>\n");
	fprintf(f, "<p align=\"left\"><font face=\"%s\">", pi.font.c_str());
	fprintf(f, "Click on the thumbnails to get the full-size image. ");
	int x, y;
	bool all_equal_sizes = true;
	i = begin();
	x = (*i)->get_original_x(); y = (*i)->get_original_y();
	
	for (i++; i != end(); i++) {
		if ((*i)->get_original_x() != x || (*i)->get_original_y() != y) {
			all_equal_sizes = false;
			break;
		}
	}
	scale_sizes(x, y, pi.PicSize);
	if (all_equal_sizes)
		fprintf(f, "All images are either %dx%d or %dx%d.", x, y, y, x);
	fprintf(f, "</font></p>\n");
	fprintf(f, "<p align=\"left\">&nbsp;</p>\n");
	fprintf(f, "<hr>\n");
	fprintf(f, "<p align=\"right\"><font size=\"1\">&copy; anriikata's ");
	fprintf(f, "<a href=\"http://aggen.sourceforge.net/\">gallery page");
	fprintf(f, " creator, version %s</a></font></p>\n", VERSION);

	fprintf(f, "</body>\n</html>\n");

	fclose(f);
	return true;
}

void Project:: WriteJavaScript(FILE *f)
{
	#define o fprintf
	o(f, "<script language=\"JavaScript\">\n");
	o(f, "<!--\n");
	o(f, "function MM_swapImgRestore() { //v3.0\n");
	o(f, "  var i,x,a=document.MM_sr; for(i=0;a&&i<a.length&&(x=a[i])&&x.oSrc;i++) x.src=x.oSrc;\n");
	o(f, "}\n\n");
	o(f, "function MM_preloadImages() { //v3.0\n");
	o(f, "  var d=document; if(d.images){ if(!d.MM_p) d.MM_p=new Array();\n");
	o(f, "    var i,j=d.MM_p.length,a=MM_preloadImages.arguments; for(i=0; i<a.length; i++)\n");
	o(f, "    if (a[i].indexOf(\"#\")!=0){ d.MM_p[j]=new Image; d.MM_p[j++].src=a[i];}}\n");
	o(f, "}\n\n");
	o(f, "function MM_findObj(n, d) { //v4.0\n");
	o(f, "  var p,i,x;  if(!d) d=document; if((p=n.indexOf(\"?\"))>0&&parent.frames.length) {\n");
	o(f, "    d=parent.frames[n.substring(p+1)].document; n=n.substring(0,p);}\n");
	o(f, "  if(!(x=d[n])&&d.all) x=d.all[n]; for (i=0;!x&&i<d.forms.length;i++) x=d.forms[i][n];\n");
	o(f, "  for(i=0;!x&&d.layers&&i<d.layers.length;i++) x=MM_findObj(n,d.layers[i].document);\n");
	o(f, "  if(!x && document.getElementById) x=document.getElementById(n); return x;\n");
	o(f, "}\n\n");
	o(f, "function MM_swapImage() { //v3.0\n");
	o(f, "  var i,j=0,x,a=MM_swapImage.arguments; document.MM_sr=new Array; for(i=0;i<(a.length-2);i+=3)\n");
	o(f, "   if ((x=MM_findObj(a[i]))!=null){document.MM_sr[j++]=x; if(!x.oSrc) x.oSrc=x.src; x.src=a[i+2];}\n");
	o(f, "}\n//-->\n</script>\n");
	#undef o
}


bool Project:: CreateHtml(void)
{
	CheckExifStats();
	if (pi.galleryType == GT_PAGE_WITH_THUMBS) {
		int gc;
		if (pi.PicsPerGallery) {
			gc = (GetCount() - 1) / pi.PicsPerGallery + 1;
		} else {
			gc = 1;
		}
		for (int i = 0; i < gc; i++) {
			bool r = CreateHtmlPage(i, gc);
			if (!r) return false;
		}
		return true;
	} else {
		return CreateJavaScriptPage();
	}
}

bool Project:: CreateJavaScriptPage(void)
{
	//Misho, implement this!
	printf("Blaaablabla!");
	return true;
}

void Project:: CheckExifStats(void)
{
	have_exif_stats = false;
	if (pi.exif_stats && GetCount() > 1) {
		// let's try to gather statistics then...
		vector<Rational> exposures;
		vector<float> apertures;
		vector<int> ISOs;
		vector<int> focals_real;
		vector<int> focals_35mm;
		bool use_35mm = true;
		//
		for (ProjectIterator i = begin(); i != end(); i++) {
			RawImg *img = *i;
			ExifWrapper *exif = img->get_exif();
			if (!exif) continue;
			Rational r; float f; int iso; int focal;
			if (exif->get_exposure(r))
				exposures.push_back(r);
			if (exif->get_aperture(f))
				apertures.push_back(f);
			if (exif->get_iso(iso))
				ISOs.push_back(iso);
			if (exif->get_focal_length(focal)) {
				focals_real.push_back(focal);
				if (exif->get_effective_focal_length(focal)) {
					focals_35mm.push_back(focal);
				} else {
					use_35mm = false;
				}
			}
		}
		// Check what we've gathered:
		if (exposures.size() <= 1 && apertures.size() <= 1 && ISOs.size() <= 1
			&& focals_real.size() <= 1 && focals_35mm.size() <= 1) return;
		// OK, we've got material for statistics
	}
}


FILE* Project:: CreateExifStatsPageHeader(void)
{
	return NULL;
}

void Project:: AddExposureData(FILE*, Rational *, int)
{
}

void Project:: AddApertureData(FILE*, float *, int)
{
}

void Project:: AddISOData(FILE*, int *, int)
{
}

void Project:: AddFocalLengthData(FILE *, int *, int, bool)
{
}
