/* Created by Anjuta version 1.2.3 */
/*	This file will not be overwritten */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <wx/wx.h>
#include <wx/image.h>
#include <wx/filename.h> 


#include "tools.h"
#include "defaults.h"
#include "project.h"
#include "configuration.h"
#include "preferences.h"
#include "noteswnd.h"
#include "prefsdlg.h"
#include "exifinfodlg.h"
#ifndef __WIN32__
#include "aggicon.xpm"
#endif

class MyApp : public wxApp
{
public:
	virtual bool OnInit();
};

wxWindow * globMainFrame;

class MainFrame : public wxFrame {
public:
	MainFrame(const wxString &, const wxPoint &, const wxSize &);
	void OnHPaint(wxPaintEvent & event);
	void OnReSize(wxSizeEvent & event);
	void HandleResize(void);
	void OnNewProject(wxCommandEvent & event);
	void OnOpenProject(wxCommandEvent & event);
	void OnSaveProject(wxCommandEvent & event);
	void OnSaveProjectAs(wxCommandEvent & event);
	void OnMenuPreferences(wxCommandEvent & event);
	void OnMenuExit(wxCommandEvent & event);
	void OnRenameAnnotate(wxCommandEvent & event);
	void OnExifInfo(wxCommandEvent & event);
	void OnQuickExifInfo(wxCommandEvent & event);
	void OnConfigure(wxCommandEvent & event);
	void OnHelpAbout(wxCommandEvent & event);
	void Init(void);
	void OnPrevClick(wxCommandEvent& event);
	void OnRotClick(wxCommandEvent& event);
	void OnNextClick(wxCommandEvent& event);
	void OnSaveClick(wxCommandEvent& event);
	void OnGenerateClick(wxCommandEvent& event);
	void OnGeneratePreview(wxCommandEvent& event);
	void OnHasFrameClick(wxCommandEvent& event);
	void OnIncludedClick(wxCommandEvent& event);
	
	// shows the Rename/Annotate wnd
	void ShowNotes(void);
	
	// shows the Exif info wnd
	void ShowExifInfo(void);
	void ShowQuickExifInfo(void);

	// mouse events
	void OnLeftUp(wxMouseEvent& event);
	void OnRightUp(wxMouseEvent & event);
	void OnMouseWheel(wxMouseEvent & event);
	bool IsInPreviewRange(wxMouseEvent & event);
	void GetPreviewOffset(int xsize, int ysize, int& X, int& Y);

	void HandleKeyboard(wxKeyEvent & event);

	void SaveDisable(void);
	void SaveEnable(void);

	void ActDeactMenu(int, int, bool);
	void ActivateProj(bool);
	void ActivatePreview(void);
	void DeactivatePreview(void);
	void UpdatePreview(wxPaintDC&);

	int SaveProject(void);
	void OpenProject(wxString & FN);

private:
	wxColour BGCol;
	bool isActive, MessageOnBar;
	wxStatusBar *StatusBar;
	wxButton *PrevImg, *RotImg, *NextImg, *SaveButton, *GenerateButton;
	wxCheckBox *HasFrame;
	wxCheckBox *Included;
	DECLARE_EVENT_TABLE()

};

ProjectInfo ProjInfo;
Project CurrentProject;

// IDs for the menu items
enum {
	Main_File_NewProject = 1,
	Main_File_OpenProject,
	Main_File_SaveProject,
	Main_File_SaveProjectAs,
	Main_File_Preferences,
	Main_File_Exit,
	Main_Picture_RenameAnnotate,
	Main_Picture_ExifInfo,
	Main_Picture_QuickExifInfo,
	Main_Gallery_Configure,
	Main_Gallery_Preview,
	Main_Help_About,
	Main_PrevImg_Button,
	Main_RotImg_Button,
	Main_NextImg_Button,
	Main_Save_Button,
	Main_Generate_Button,
	Main_HasFrame_Box,
	Main_Included_Box
};

enum {
	E_DIDNT_SAVE,
	E_OK,
	E_NOT_CHOSEN
};

BEGIN_EVENT_TABLE(MainFrame, wxFrame)
	EVT_MENU(Main_File_NewProject, MainFrame:: OnNewProject)
	EVT_MENU(Main_File_OpenProject,MainFrame:: OnOpenProject)
	EVT_MENU(Main_File_SaveProject,MainFrame:: OnSaveProject)
	EVT_MENU(Main_File_SaveProjectAs, MainFrame:: OnSaveProjectAs)
	EVT_MENU(Main_File_Preferences, MainFrame:: OnMenuPreferences)
	EVT_MENU(Main_File_Exit, MainFrame:: OnMenuExit)
	EVT_MENU(Main_Picture_RenameAnnotate, MainFrame:: OnRenameAnnotate)
	EVT_MENU(Main_Picture_ExifInfo, MainFrame:: OnExifInfo)
	EVT_MENU(Main_Picture_QuickExifInfo, MainFrame:: OnQuickExifInfo)
	EVT_MENU(Main_Gallery_Configure, MainFrame:: OnConfigure)
	EVT_MENU(Main_Gallery_Preview, MainFrame:: OnGeneratePreview)
	EVT_MENU(Main_Help_About, MainFrame:: OnHelpAbout)
	EVT_BUTTON(Main_PrevImg_Button, MainFrame:: OnPrevClick)
	EVT_BUTTON(Main_RotImg_Button, MainFrame:: OnRotClick)
	EVT_BUTTON(Main_NextImg_Button, MainFrame:: OnNextClick)
	EVT_BUTTON(wxID_SAVE, MainFrame:: OnSaveClick)
	EVT_BUTTON(Main_Generate_Button, MainFrame:: OnGenerateClick)
	EVT_KEY_UP(MainFrame:: HandleKeyboard)
	EVT_LEFT_UP(MainFrame:: OnLeftUp)
	EVT_RIGHT_UP(MainFrame:: OnRightUp)
	EVT_MOUSEWHEEL(MainFrame:: OnMouseWheel)
	EVT_CHECKBOX(Main_HasFrame_Box, MainFrame:: OnHasFrameClick)
	EVT_CHECKBOX(Main_Included_Box, MainFrame:: OnIncludedClick)
	EVT_PAINT(MainFrame:: OnHPaint)
	EVT_SIZE(MainFrame:: OnReSize)
END_EVENT_TABLE()

MainFrame:: MainFrame(const wxString& title, const wxPoint& pos, 
	const wxSize& size)
	: wxFrame((wxFrame *)NULL, wxID_ANY, title, pos, size)
{
	globMainFrame = this;
	wxPanel *panel = new wxPanel(this);
	// an ugly hack to use no panel even under windoze :)
	BGCol = panel->GetBackgroundColour();
	panel->Destroy();
	Init();
}

void MainFrame:: Init(void)
{
	/* Set Frame icon */
	SetIcon(wxICON(aggicon));

	wxMenuBar *menu = new wxMenuBar;
	if (menu) {
		wxMenu *FileMenu = new wxMenu;
		FileMenu->Append(Main_File_NewProject, wxT("&New Project...\tCtrl-N"));
		FileMenu->Append(Main_File_OpenProject, wxT("&Open Project...\tCtrl-O"));
		FileMenu->Append(Main_File_SaveProject, wxT("&Save Project\tCtrl-S"));
		FileMenu->Append(Main_File_SaveProjectAs, wxT("Save Project &As...\tCtrl-A"));
		FileMenu->Append(Main_File_Preferences, wxT("&Preferences..."));
		FileMenu->AppendSeparator();
		FileMenu->Append(Main_File_Exit, wxT("E&xit\tAlt-X"));
		menu->Append(FileMenu, wxT("&File"));
		wxMenu *PictureMenu = new wxMenu;
		PictureMenu->Append(Main_Picture_RenameAnnotate, 
			wxT("&Rename/Annotate...\tF2"));
		PictureMenu->Append(Main_Picture_ExifInfo,
			wxT("Exif &info...\tF3"));
		PictureMenu->Append(Main_Picture_QuickExifInfo,
			wxT("&Quick Exif info\tF4"));
		menu->Append(PictureMenu, wxT("&Picture"));
		wxMenu *GalleryMenu = new wxMenu;
		GalleryMenu->Append(Main_Gallery_Configure, wxT("&Configure...\tAlt-C"));
		GalleryMenu->Append(Main_Gallery_Preview, wxT("Create P&review...\tCtrl-P"));
		menu->Append(GalleryMenu, wxT("&Gallery"));
		wxMenu *HelpMenu = new wxMenu;
		HelpMenu->Append(Main_Help_About, wxT("&About"));
		menu->Append(HelpMenu, wxT("&Help"));
	}
	SetMenuBar(menu);
		
	ActDeactMenu(1, 0, false);
	ActDeactMenu(2, 0, false);
	ActDeactMenu(2, 1, false);
	ActDeactMenu(0, 2, false);
	ActDeactMenu(0, 3, false);


	StatusBar = CreateStatusBar(2);
	int widths[2] = {-1, 140};
	StatusBar->SetStatusWidths(2, widths);
	if (StatusBar) {
		StatusBar->SetStatusText(wxT("No Project Loaded"), 0);
	}
	PrevImg = new wxButton(this, Main_PrevImg_Button, wxT("&Prev"), 
		wxPoint(BTN_X_BEGIN +  0, 20), wxSize(60, 36), wxNO_BORDER);
	RotImg = new wxButton(this, Main_RotImg_Button, wxT("&Rotate CCW"),
		wxPoint(BTN_X_BEGIN + 60, 20), wxSize(100, 36), wxNO_BORDER);
	NextImg = new wxButton(this, Main_NextImg_Button, wxT("&Next"),
		wxPoint(BTN_X_BEGIN +160, 20), wxSize(60, 36), wxNO_BORDER);
	SaveButton = new wxButton(this, wxID_SAVE, "", 
		wxPoint(BTN_X_BEGIN + 228, 20), wxSize(60, 36));
	GenerateButton = new wxButton(this, Main_Generate_Button,
		wxT("&Generate Gallery"),
		wxPoint(BTN_X_BEGIN + 290, 20), wxSize(120, 36));
	SaveDisable();
	GenerateButton->Disable();
	HasFrame = new wxCheckBox(this, Main_HasFrame_Box,
		wxT("&Thumbnail Frame Fadeout"));
	Included = new wxCheckBox(this, Main_Included_Box,
		wxT("&Exclude from gallery"), wxPoint(EndX(HasFrame) + 10, -1));
	HandleResize();
	SetOwnBackgroundColour(BGCol);
	Show(TRUE);
	Centre();
	SetFocus();
	isActive = false;
	MessageOnBar = false;
	ProjInfo.Defaults();
}

void MainFrame:: OnNewProject(wxCommandEvent & event)
{
	ConfigurationDialog *pdlg = new ConfigurationDialog(true);
	int res = pdlg->ShowModal();
	pdlg->Destroy();
	if (0 == res) {
		DeactivatePreview();
		ProjInfo.has_frame = DEFAULT_HAS_FRAME;
		if (!CurrentProject.NewProject(ProjInfo))
			return;
		CurrentProject.SetCurrent(0);
		wxString tempfn = wxFileName::CreateTempFileName(wxT(""));
		CurrentProject.SaveToFile(tempfn);
		CurrentProject.Destroy();
		// reload project, creating thumbnails
		if (CurrentProject.LoadFromFile(tempfn, true, true /*guess orientations*/)) {
			wxMessageBox(wxT("Project created successfully; # of pictures: ") + 
				wxString::Format(wxT("%d"), CurrentProject.size()));
		
			HasFrame->SetValue(CurrentProject.GetHasFrame());
			

			CurrentProject.SetCurrent(0);
			Included->SetValue(!CurrentProject.GetCurrentState());
			CurrentProject.FilenameToSave = wxEmptyString;
			CurrentProject.MustSave = true;
		
			ActivatePreview();
			ActivateProj(true);
		}
		wxRemoveFile(tempfn);
	}
}

void MainFrame:: OpenProject(wxString & FN)
{
	CurrentProject.FilenameToSave = FN;
	if (CurrentProject.LoadFromFile(FN, true)) {
		HasFrame->SetValue(CurrentProject.GetHasFrame());
		CurrentProject.SetCurrent(0);
		ProjInfo = CurrentProject.GetProjectInfo();
		ActivatePreview();
		MessageOnBar = true;
		StatusBar->SetStatusText(wxT("Project loaded successfully"));
		ActivateProj(false);
		GenerateButton->Enable();
			
	} else {
		wxMessageBox(wxT("Error occured while loading the project"),
			wxT("Error"), wxICON_ERROR);
	}
}

void MainFrame:: OnOpenProject(wxCommandEvent & event)
{
	wxString FN = wxFileSelector(wxT("Open a project"), wxT(""), wxT(""), wxT("*.agg"),
		wxT("Anrieff Gallery Projects (*.agg)|*.agg"),
		wxOPEN|wxFILE_MUST_EXIST);
	if (!FN.empty()) {
		DeactivatePreview();
		CurrentProject.Destroy();
		OpenProject(FN);
	}
}
void MainFrame:: OnSaveProject(wxCommandEvent & event)
{
	if (CurrentProject.MustSave) {
		if (SaveProject()!=E_DIDNT_SAVE) {
			MessageOnBar = true;
			StatusBar->SetStatusText(wxT("Project Saved: \"")+
				CurrentProject.FilenameToSave +wxT("\""));
		} else {
			wxMessageBox(wxT("Error occured while saving the project"),
				wxT("Error"), wxICON_ERROR);
		}
	}
}
void MainFrame:: OnSaveProjectAs(wxCommandEvent & event)
{
	if (ChooseFileName(&CurrentProject)) {
		if (CurrentProject.SaveToFile(CurrentProject.FilenameToSave)) {
			CurrentProject.MustSave = false;
			MessageOnBar = true;
			StatusBar->SetStatusText(wxT("Project Saved: \"")+
				CurrentProject.FilenameToSave +wxT("\""));
		} else {
			wxMessageBox(wxT("Error occured while saving the project"),
				wxT("Error"), wxICON_ERROR);
		}
	}
}
void MainFrame:: OnMenuPreferences(wxCommandEvent & event)
{
	PrefsDialog * prefsdlg = new PrefsDialog(this);
	prefsdlg->ShowModal();
	prefsdlg->Destroy();
}
void MainFrame:: OnMenuExit(wxCommandEvent & event)
{
	Close();
}
void MainFrame:: OnRenameAnnotate(wxCommandEvent & event)
{
	ShowNotes();
}
void MainFrame:: OnExifInfo(wxCommandEvent & event)
{
	ShowExifInfo();
}
void MainFrame:: OnQuickExifInfo(wxCommandEvent & event)
{
	ShowQuickExifInfo();
}
void MainFrame:: ShowNotes(void)
{
	NotesDialog * pdlg = new NotesDialog(CurrentProject.GetCurrentImg(), 
		ProjInfo.PicSize);
	int res = pdlg->ShowModal();
	pdlg->Destroy();
	if (res == 0) {
		CurrentProject.MustSave = true;
		SaveEnable();
	}
}

void MainFrame:: ShowExifInfo(void)
{
	ExifInfoDlg * pdlg = new ExifInfoDlg(this, CurrentProject.GetCurrentImg());
	pdlg->ShowModal();
	pdlg->Destroy();
}

void MainFrame:: ShowQuickExifInfo(void)
{
	RawImg *img = CurrentProject.GetCurrentImg();
	ExifWrapper *exif = img->get_exif();
	if (!exif) {
		wxMessageBox(wxT("No exif"));
		return;
	}
	wxString exposure = "N/A", aperture = "N/A", iso = "N/A", flen = "N/A";
	Rational r;
	float f;
	int i;
	int len;
	if (exif->get_exposure(r)) {
		exposure = wxString::Format("%d/%d", r.num, r.den);
	}
	if (exif->get_aperture(f)) {
		aperture = wxString::Format("f/%.1f", f);
	}
	if (exif->get_iso(i)) {
		iso = wxString::Format("%d", i);
	}
	if (exif->get_effective_focal_length(len)) {
		flen = wxString::Format("%dmm (35mm film equiv.)", len);
	} else {
		if (exif->get_focal_length(len)) {
			flen = wxString::Format("%dmm", len);
		}
	}
	wxMessageBox(
	wxString(wxT("Exposure: ")) + exposure + wxT("\n")
	        +wxT("Aperture: ")  + aperture + wxT("\n")
	        +wxT("ISO Speed: ") + iso      + wxT("\n")
	        +wxT("Focal length: ") + flen,
				wxT("Quick Exif info"));
}

void MainFrame:: OnHPaint(wxPaintEvent & event)
{
	wxPaintDC dc(this);
    UpdatePreview(dc);
}
void MainFrame:: OnReSize(wxSizeEvent & event)
{
	HandleResize();
}
void MainFrame:: HandleResize(void)
{
	static int firstresize=0;
	if (firstresize) {
		wxSize cSize = GetClientSize();
		wxRect rect  = HasFrame->GetRect();
		int yy = cSize.GetHeight() - rect.height - FORM_BRDR;
		HasFrame->SetSize(4, yy, -1, -1);
		Included->SetSize(-1, yy, -1, -1);
	} else {
		firstresize = 1;
	}
}

void MainFrame:: OnPrevClick(wxCommandEvent& event)
{
	CurrentProject.AdjustCurrent(-1);
	Refresh();
	SetFocus();
}
void MainFrame:: OnRotClick(wxCommandEvent& event)
{
	RawImg *t = CurrentProject.GetCurrentImg();
	t->set_rotation(t->get_rotation()+1);
	//UpdatePreview();
	Refresh();
	CurrentProject.MustSave = true;
	SaveEnable();
	SetFocus();
}
void MainFrame:: OnNextClick(wxCommandEvent& event)
{
	CurrentProject.AdjustCurrent(+1);
	Refresh();
	SetFocus();
}

// mouse equivvalents of the above fns
void MainFrame:: OnLeftUp(wxMouseEvent& event)
{
	if (!isActive) return;
	if (IsInPreviewRange(event)) {
		wxCommandEvent cmd;
		if (preferences.swap_lr)
			OnRotClick(cmd);
		else
			OnNextClick(cmd);
	}
}
void MainFrame:: OnRightUp(wxMouseEvent & event)
{
	if (!isActive) return;
	if (IsInPreviewRange(event)) {
		wxCommandEvent cmd;
		if (preferences.swap_lr)
			OnNextClick(cmd);
		else
			OnRotClick(cmd);
	}
}
void MainFrame:: OnMouseWheel(wxMouseEvent & event)
{
	if (!isActive) return;
	int delta = event.GetWheelRotation() / event.GetWheelDelta();
	CurrentProject.AdjustCurrent(-delta);
	Refresh();
}

bool MainFrame:: IsInPreviewRange(wxMouseEvent & event)
{
	if (!isActive) return false;
	RawImg *t = CurrentProject.GetCurrentImg();
	int x, y, xs, ys;
	t->get_rotated_sizes(xs, ys);
	GetPreviewOffset(xs, ys, x, y);

	x = event.m_x - x;
	y = event.m_y - y;

	return (x < xs && y < ys && x >= 0 && y >= 0);
}

void MainFrame:: OnSaveClick(wxCommandEvent& event)
{
	if (SaveProject()!=E_DIDNT_SAVE) {
		MessageOnBar = true;
		StatusBar->SetStatusText(wxT("Project Saved: \"")+
			CurrentProject.FilenameToSave +wxT("\""));
	} else {
		wxMessageBox(wxT("Error occured while saving the project"),
				wxT("Error"), wxICON_ERROR);
	}
	SetFocus();
}
void MainFrame:: OnGenerateClick(wxCommandEvent& event)
{
	CurrentProject.CreateGallery(true);
	SetFocus();
}

void MainFrame:: OnGeneratePreview(wxCommandEvent& event)
{
	if (!isActive || !GenerateButton->IsEnabled())
		return;
	CurrentProject.CreateGallery(false);
}


void MainFrame:: OnHasFrameClick(wxCommandEvent& event)
{
	CurrentProject.SetHasFrame(HasFrame->GetValue());
	CurrentProject.MustSave = true;
	SaveEnable();
	Refresh();
}

void MainFrame:: OnIncludedClick(wxCommandEvent& event)
{
	CurrentProject.SetCurrentState(!Included->GetValue());
	CurrentProject.MustSave = true;
	SaveEnable();
}

void MainFrame:: ActDeactMenu(int menu, int item, bool state)
{
	wxMenu * m = GetMenuBar()->GetMenu(menu);
	if (m) {
		wxMenuItem *i = m->FindItemByPosition(item);
		if (i) {
			if (state)
				i->Enable();
			else
				i->Enable(false);
		}
	}
}

void MainFrame:: ActivateProj(bool withsave)
{
	GenerateButton->Enable();
	
	ActDeactMenu(1, 0, true);
	ActDeactMenu(2, 0, true);
	ActDeactMenu(2, 1, true);
	ActDeactMenu(0, 3, true);
	if (withsave) {
		SaveButton->Enable();
		ActDeactMenu(0, 2, true);
	}
}

void MainFrame:: ActivatePreview(void)
{
	isActive = true;
	PrevImg->Enable();
	RotImg->Enable();
	NextImg->Enable();
	HasFrame->Enable();
	Included->Enable();
	Refresh();
}
void MainFrame:: DeactivatePreview(void)
{
	isActive = false;
	PrevImg->Disable();
	RotImg->Disable();
	NextImg->Disable();
	HasFrame->Disable();
	Included->Disable();
}
void MainFrame:: UpdatePreview(wxPaintDC & dc)
{
	if (!isActive) {
		return;
	}
	Included->SetValue(!CurrentProject.GetCurrentState());
	RawImg *cc = CurrentProject.GetCurrentImg();
	if (MessageOnBar) {
		MessageOnBar = false;
		StatusBar->SetStatusText(wxEmptyString, 1);
	} else {
		StatusBar->SetStatusText(
			wxString::Format(wxT("[%d/%d]: "),1+ CurrentProject.GetCurrentID(), 
			CurrentProject.size()) + cc->FileName, 0);
		StatusBar->SetStatusText(wxString::Format(wxT("%dx%d pixels"), 
			cc->get_original_x(), cc->get_original_y()), 1);
	}
	unsigned char* prev = cc->CreateRGBPreview(HasFrame->GetValue(), 
		BgColorRGB(this));
	int imgx, imgy;
	cc->get_rotated_sizes(imgx, imgy);
	wxImage img(imgx, imgy, (unsigned char*) prev);
	if (!img.Ok())
		wxMessageBox(wxT("Bad Image"));
	
	wxBitmap bmp(img);
	if (!bmp.Ok())
		wxMessageBox(wxT("Bad Bitmap"));
	else {
		int Xoffset, Yoffset;
		GetPreviewOffset(imgx, imgy, Xoffset, Yoffset);
		dc.DrawBitmap(bmp, Xoffset, Yoffset, false);
	}
}

void MainFrame:: GetPreviewOffset(int xsize, int ysize, int & X, int & Y)
{
	X = BTN_X_BEGIN;
	if (xsize < BTN_X_SIZE) {
		X += (BTN_X_SIZE - xsize)/2;
	}
	Y = 60;
}

int MainFrame:: SaveProject(void)
{
	if (CurrentProject.FilenameToSave == wxEmptyString) {
		bool chose = ChooseFileName(&CurrentProject);
		if (!chose)
			return E_NOT_CHOSEN;
	}
	if (CurrentProject.SaveToFile(CurrentProject.FilenameToSave)) {
		CurrentProject.MustSave = false;
		SaveDisable();
		return E_OK;
	} else return E_DIDNT_SAVE;
}

void MainFrame:: SaveDisable(void)
{
	SaveButton->Disable();
	ActDeactMenu(0, 2, false);
}
void MainFrame:: SaveEnable(void)
{
	SaveButton->Enable();
	ActDeactMenu(0, 2, true);
}


void MainFrame:: OnConfigure(wxCommandEvent & event)
{
	//
	if (!isActive) return;
	
	int oldThumbSize = ProjInfo.ThumbSize;
	ConfigurationDialog *pdlg = new ConfigurationDialog(false);
	int res = pdlg->ShowModal();
	pdlg->Destroy();
	if (0 == res) {
		CurrentProject.SetProjectInfo(ProjInfo);
		oldThumbSize -= ProjInfo.ThumbSize;
		if (oldThumbSize) {// needs recreating the thumbnails
			DeactivatePreview();
			CurrentProject.FreeImageData();
			CurrentProject.RecreateThumbs();
			ActivatePreview();
		}
		CurrentProject.MustSave = true;
		SaveEnable();
	}
}

void MainFrame:: HandleKeyboard(wxKeyEvent & event)
{
	
	if (!isActive) {
		event.Skip();
		return;
	}
	switch (event.GetKeyCode()) {
		case WXK_RIGHT:
		case WXK_NUMPAD6:
		case WXK_PAGEDOWN:
			{
				wxCommandEvent cmd;
				OnNextClick(cmd);
				break;
			}
		case WXK_LEFT:
		case WXK_NUMPAD4:
		case WXK_PAGEUP:
			{
				wxCommandEvent cmd;
				OnPrevClick(cmd);
				break;
			}
		case 'I':
			{
				ShowExifInfo();
				break;
			}
		default: event.Skip();
	}
}

void MainFrame:: OnHelpAbout(wxCommandEvent & event)
{
	wxDialog *AboutBox = new wxDialog(this, -1, wxT("About"), wxDefaultPosition,
		wxSize(320, 306));
	int x, y;
	AboutBox->GetClientSize(&x, &y);
	wxButton *okok = new wxButton(AboutBox, wxID_OK);
	okok->SetSize(x/2-40, y-40, 80, 30);
	
	wxIcon ico(wxICON(aggicon));
	wxBitmap logo(ico);
	wxStaticBitmap *sbmp = new wxStaticBitmap(AboutBox, -1, logo, 
		wxPoint((x-logo.GetWidth())/2, 25));
	sbmp->Refresh();
	
	wxStaticText *aggt = new wxStaticText(AboutBox, -1, 
		wxT("Anrieff's Gallery Generator"), wxPoint(1,  76));
	wxFont txtFont = aggt->GetFont();
	txtFont.SetWeight(wxFONTWEIGHT_BOLD);
	aggt->SetFont(txtFont);
	aggt->SetSize((x-EndX(aggt))/2, -1, -1, -1);
	wxStaticText *aggv = new wxStaticText(AboutBox, -1,
		wxString::Format(wxT("Version %s"), VERSION), 
		wxPoint(0, EndY(aggt)+2));
	aggv->SetSize((x-EndX(aggv))/2, -1, -1, -1);
	wxStaticText *aggs = new wxStaticText(AboutBox, -1,
		wxT("http://aggen.sourceforge.net/"), wxPoint(0, EndY(aggv)+2));
	aggs->SetSize((x-EndX(aggs))/2, -1, -1, -1);

	
	wxStaticText *txt1 = new wxStaticText(AboutBox, -1, 
		//wxT("Authors\nE-Mail\n\nThanks to"),
		wxT("Author\nE-Mail\n\nThanks to"),
		wxPoint(10,  140));
	txtFont = txt1->GetFont();
	txtFont.SetWeight(wxFONTWEIGHT_BOLD);
	txt1->SetFont(txtFont);
	wxStaticText *txt2 = new wxStaticText(AboutBox, -1,
		//wxT("Veselin Georgiev, Mihail Stanchev\nvessko_AT_SoftHome_D0T_net\n\nwxWidgets\nIndependent JPEG Group"),
		wxT("Veselin Georgiev\nvessko_AT_SoftHome_D0T_net\n\nwxWidgets\nIndependent JPEG Group\nlibexif"),
		wxPoint(95,  140));
	txt2->Refresh();
	
	AboutBox->ShowModal();
}

IMPLEMENT_APP(MyApp)

bool MyApp::OnInit()
{
	preferences.load();
	MainFrame *main_frame = new MainFrame(
				wxT("AGG - Anrieff's Gallery Generator"),
				wxPoint(50, 50), 
				wxSize(500, 340)
			);
	main_frame->DeactivatePreview();
	main_frame->Refresh();
	if (argc == 2) {
		wxString fn = argv[1];
		if (wxFileExists(fn))
			main_frame->OpenProject(fn);
	}
  	return TRUE;
}
