/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "noteswnd.h"
#include "tools.h"
#include "main.h"
#include "defaults.h"
#include <wx/numdlg.h>

int NotesDialog::resolutions[16] = {0};
int NotesDialog::res_count = 1;

BEGIN_EVENT_TABLE(NotesDialog, wxDialog)
	EVT_BUTTON(wxID_OK, NotesDialog:: OnOKClick)
	EVT_BUTTON(Notes_Default, NotesDialog:: OnDefaultsClick)
	EVT_TEXT_ENTER(Notes_Text1, NotesDialog:: PushValue)
	EVT_TEXT(Notes_Text1, NotesDialog:: CheckFN)
	EVT_COMBOBOX(ComboBox_Res, NotesDialog:: OnResChanged)
END_EVENT_TABLE()

NotesDialog:: NotesDialog(RawImg* rimg, int defres) 
		: wxDialog(globMainFrame, 
		wxID_ANY, 
		wxT("Rename / Annotate File"),
		wxDefaultPosition,
		wxSize(350, 145+WINDOWSVADD)
		)
{
	this->defres = defres;
	fn_ctrl = NULL;
	img = rimg;
	wxStaticText *tt1 = new wxStaticText (this, -1, wxT("File Name:"));
	tt1->SetSize(20, 20, -1, -1);
	//
	int img_res = img->get_resolution();
	if (img_res != 0) {
		int i = 0;
		while (i < res_count && resolutions[i] != img_res) i++;
		if (i >= res_count) resolutions[res_count++] = img_res;
	}
	//
	wxStaticText *ttr = new wxStaticText (this, -1, wxT("Resized resolution:"));
	ttr->SetSize(20, 60, -1, -1);
	wxArrayString values = GetResolutionChoices(rimg);
	resolution = new wxComboBox(this, ComboBox_Res, values[0], 
	    wxPoint(EndX(ttr)+5, 56),	wxDefaultSize, values, 
		wxCB_DROPDOWN | wxCB_READONLY);
	resolution->SetSize(-1,-1,165,-1);
	if (img_res)
		resolution->SetValue(GetTextFor(img_res));
	//
	button_OK = new wxButton(this, wxID_OK);
	button_OK->SetSize(20, 105, 80, 30);
	wxButton *button_Cancel = new wxButton(this, wxID_CANCEL);
	button_Cancel->SetSize(110, 105, 80, 30);
	button_Defaults = new wxButton(this, Notes_Default, wxT("&Defaults"), 
		wxPoint(220, 105), wxSize(80, 30));
	fn_ctrl = new wxTextCtrl(this, Notes_Text1, 
		stripExtension(img->outFileName), wxPoint(5 + EndX(tt1), 16),
		wxSize(200, 24), wxTE_PROCESS_ENTER);
	wxStaticText *tt2 = new wxStaticText(this, -1, wxT(".jpg"), 
		wxPoint(2+EndX(fn_ctrl), 20));
	tt2->Refresh();


	fn_ctrl->SetSelection(-1,-1);
	fn_ctrl->SetFocus();

}


void NotesDialog:: OnOKClick(wxCommandEvent & event)
{
	img->outFileName = fn_ctrl->GetValue() + wxT(".jpg");
	img->set_resolution(resolutions[resolution->FindString(resolution->GetValue())]);
	EndModal(0);
}

void NotesDialog:: OnDefaultsClick(wxCommandEvent & event)
{
	fn_ctrl->SetValue(stripExtension(GetFn(img->FileName)));
	fn_ctrl->SetSelection(-1,-1);
	fn_ctrl->SetFocus();
	resolution->SetSelection(0);
}

void NotesDialog:: PushValue(wxCommandEvent & event)
{
	if (button_OK->IsEnabled()) {
		img->outFileName = fn_ctrl->GetValue() + wxT(".jpg");
		img->set_resolution(resolutions[resolution->FindString(resolution->GetValue())]);
		EndModal(0);
	}
}

void NotesDialog:: CheckFN(wxCommandEvent & event)
{
	if (!fn_ctrl) return;
	wxString v = fn_ctrl->GetValue();
	bool ok = (!v.empty());
	for (unsigned i = 0; i < v.length(); i++) {
		char c = v[i];
		if (c == '/' || c == '\\' || c == ':' || c == '?' || c == '*' 
			|| c == '>' || c == '<' || c == '|') {
				ok = false;
				break;
			}
	}
	if (ok) {
		button_OK->Enable();
	} else {
		button_OK->Enable(false);
	}
}

wxArrayString NotesDialog:: GetResolutionChoices(RawImg *img)
{
	wxArrayString res;
	for (int i = 0; i < res_count; i++) {
		wxString s = GetTextFor(resolutions[i]);
		if (i == 0) {
			s = wxString(wxT("Default (")) + s + wxT(")");
		}
		res.Add(s);
	}
	res.Add(wxString("Custom..."));
	return res;
}

void NotesDialog:: OnResChanged(wxCommandEvent & event)
{
	if (resolution->GetValue() == wxT("Custom...")) {
		long newres = wxGetNumberFromUser(
			wxT("Please, enter the length of the longer side\n")
		    wxT("(or enter 9999 to leave this image unscaled)"),
		wxT("Length:"), wxT("Input"), defres/2, 1, 9999, this);
		if (newres != -1) {
			resolutions[res_count++] = (int) newres;
			wxString new_entry = GetTextFor((int) newres);
			resolution->Insert(new_entry, resolution->FindString(wxT("Custom...")));
			resolution->SetValue(new_entry);
		}
	}
}

wxString NotesDialog:: GetTextFor(int bigger_side)
{
	if (!bigger_side)
		bigger_side = defres;
	int x, y;
	CalculateResizedSizes(*img, bigger_side, x, y, true, true);
	return wxString::Format(wxT("%dx%d"), x, y);
}
