/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <stdio.h>
#include <string.h>
#include "preferences.h"
#include "tools.h"

Preferences preferences;

void Preferences:: defaults(void)
{
	smp = true;
	swap_lr = false;
	no_sse = false;
	sort_exif = false;
	preserve_exif = true;
	custom_font_idx = 0;
	memset(custom_fonts, 0, sizeof(custom_fonts));
	memset(last_input_dir, 0, sizeof(last_input_dir));
	memset(last_output_dir, 0, sizeof(last_output_dir));
}

Preferences:: Preferences()
{
	defaults();
}

Preferences:: ~Preferences()
{
	save();
}

void Preferences:: load(void)
{
	FILE *f = fopen(PREFERENCES_SAVE_FILE, "rt");
	if (!f) return;
	char buff[1025];
	char option[1024], value[1024];
	while (fgets(buff, 1024, f)) {
		parse_pair(buff, option, value);
		if (!strcmp(option, "SMP")) {
			int x;
			if (1 == sscanf(value, "%d", &x))
				smp = x == 1 ? true : false;
		}
		if (!strcmp(option, "SwapMouseButtons")) {
			int x;
			if (1 == sscanf(value, "%d", &x)) 
				swap_lr = x==1?true:false;
		}
		if (!strcmp(option, "NoSSE")) {
			int x;
			if (1 == sscanf(value, "%d", &x)) 
				no_sse = x==1?true:false;
		}
		if (!strcmp(option, "PreserveExif")) {
			int x;
			if (1 == sscanf(value, "%d", &x)) 
				preserve_exif = x==1?true:false;
		}
		if (!strcmp(option, "SortExif")) {
			int x;
			if (1 == sscanf(value, "%d", &x)) 
				sort_exif = x==1?true:false;
		}
		if (!strncmp(option, "CustomFont", 10)) {
			int idx = option[10] - '0';
			if (idx >= 0 && idx < 4)
				strcpy(custom_fonts[idx], value);
		}
		if (!strcmp(option, "LastInputDir"))
			strcpy(last_input_dir, value);
		if (!strcmp(option, "LastOutputDir"))
			strcpy(last_output_dir, value);
	}
}

void Preferences:: save(void)
{
	FILE *f = fopen(PREFERENCES_SAVE_FILE, "wt");
	if (!f) return;
	fprintf(f, "SMP=%d\n", smp?1:0);
	fprintf(f, "SwapMouseButtons=%d\n", swap_lr?1:0);
	fprintf(f, "NoSSE=%d\n", no_sse?1:0);
	fprintf(f, "PreserveExif=%d\n", preserve_exif?1:0);
	fprintf(f, "SortExif=%d\n", sort_exif?1:0);
	for (int i = 0; i < 4; i++)
		fprintf(f, "CustomFont%d=%s\n", i, custom_fonts[i]);
	fprintf(f, "LastInputDir=%s\n", last_input_dir);
	fprintf(f, "LastOutputDir=%s\n", last_output_dir);
	fclose(f);
}
