/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "prefsdlg.h"
#include "defaults.h"
#include "tools.h"
#include "preferences.h"

BEGIN_EVENT_TABLE(PrefsDialog, wxDialog)
	EVT_BUTTON(wxID_OK, PrefsDialog:: OnOKClick)
	EVT_BUTTON(wxID_CANCEL, PrefsDialog:: OnCancelClick)
	EVT_BUTTON(PREF_Defaults, PrefsDialog:: OnDefaultsClick)
END_EVENT_TABLE()

PrefsDialog::PrefsDialog(wxWindow *parent)
: wxDialog(parent, wxID_ANY, wxT("AGG Preferences"), wxDefaultPosition, 
           wxSize(400, 220 + WINDOWSVADD))
{
	wxSize s[3];
	wxPoint p[3];
	CalculateArrangement(this, p, s, 3);
	wxButton *okb = new wxButton(this, wxID_OK, wxT(""), p[0], s[0]);
	okb->Refresh();
	wxButton *ccl = new wxButton(this, wxID_CANCEL, wxT(""), p[1], s[1]);
	ccl->Refresh();
	wxButton *def = new wxButton(this, PREF_Defaults, wxT("&Defaults"), p[2], s[2]);
	def->Refresh();
	
	cb_smp = new wxCheckBox(this, wxID_ANY, 
	                        wxT("Use multiple processors/cores (if available)"),
	                        wxPoint(10, 20), wxDefaultSize);
	cb_mouse_buttons = new wxCheckBox(this, wxID_ANY,
	                                  wxT("Swap mouse buttons for image ops"),
	                                  wxPoint(10, 50), wxDefaultSize);
	cb_sse = new wxCheckBox(this, wxID_ANY,
	                        wxT("Use advanced instruction sets (if available)"),
	                        wxPoint(10, 80), wxDefaultSize);
	cb_preserve_exif = new wxCheckBox(this, wxID_ANY,
	                        wxT("Preserve Exif metadata in output images"),
	                        wxPoint(10, 110), wxDefaultSize);
	cb_sort_exif = new wxCheckBox(this, wxID_ANY,
	                        wxT("Sort images by Exif Date (if available)"),
	                        wxPoint(10, 140), wxDefaultSize);
	ImportPrefs();
}

void PrefsDialog::OnOKClick(wxCommandEvent & event)
{
	ExportPrefs();
	EndModal(0);
}

void PrefsDialog::OnCancelClick(wxCommandEvent & event)
{
	EndModal(0);
}

void PrefsDialog::OnDefaultsClick(wxCommandEvent & event)
{
	preferences.defaults();
	ImportPrefs();
}

void PrefsDialog::ImportPrefs(void)
{
	cb_smp->SetValue(preferences.smp);
	cb_mouse_buttons->SetValue(preferences.swap_lr);
	cb_sse->SetValue(!preferences.no_sse);
	cb_sort_exif->SetValue(preferences.sort_exif);
	cb_preserve_exif->SetValue(preferences.preserve_exif);
}

void PrefsDialog::ExportPrefs(void)
{
	preferences.smp = cb_smp->GetValue();
	preferences.swap_lr = cb_mouse_buttons->GetValue();
	preferences.no_sse = !cb_sse->GetValue();
	preferences.sort_exif = cb_sort_exif->GetValue();
	preferences.preserve_exif = cb_preserve_exif->GetValue();
}
