/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
const int SSE_MASK = 0x02000000;

#include "bitmap.h"
#include "preferences.h"
 
void ArrayResize_x87(Rgb *src, int src_len, Rgb *dest, int dest_len)
{
	for (int i = 0; i < src_len; i++) {
		float x = (float) i / (src_len-1) * (dest_len-1);
		int xx = (int) x;
		float mul1 = 1.0f - (x-xx);
		dest[xx].a += mul1;
		dest[xx].b += mul1 * src[i].b;
		dest[xx].g += mul1 * src[i].g;
		dest[xx].r += mul1 * src[i].r;
		if (xx + 1 < dest_len) {
			mul1 = 1.0f - mul1;
			dest[xx + 1].a += mul1;
			dest[xx + 1].b += mul1 * src[i].b;
			dest[xx + 1].g += mul1 * src[i].g;
			dest[xx + 1].r += mul1 * src[i].r;
		}
	}
	for (int i = 0; i < dest_len; i++) {
		float mul = 1.0f / dest[i].a;
		dest[i].b *= mul;
		dest[i].g *= mul;
		dest[i].r *= mul;
		dest[i].a = 0.0f;
	}
}


#ifdef __GNUC__
/***************************************************************
 * gets some flags about the processor's capabilities          *
 *-------------------------------------------------------------*
 * This is the Windows version, requires CPUID. Should work on *
 * both AMD and Intel CPUs. Code taken from the internet:      *
 *  (looks very ugly :)                                        *
 * http://www.tommesani.com/DetectingMMXSSE.html               *
 ***************************************************************/
bool SSESupported(void) __attribute__((noinline));
bool SSESupported(void)
{
#ifdef __APPLE__
	return false;
#else
	int CPUZ=0, CPUEXT=0, ECS=0;
	 __asm __volatile (
	 "          mov     $1,    %%eax \n"
	 "          cpuid \n"
	 "          mov     %%edx,  %0\n"
	 "	    mov     $0x80000000, %%eax\n"
	 "          cpuid\n"
	 "          cmp     $0x80000001, %%eax\n"
	 "          jb      not_supported\n"
	
	 "          mov     $0x80000001, %%eax\n"
	 "          cpuid\n"
	 "          mov     %%edx,       %1\n"
	 "          movl     $1,          %2\n"
	 "          jmp     ende\n"
	
	 "          not_supported:\n"
	 "          movl     $0,          %2\n"
	 "          ende:\n"
	 :"=m"(CPUZ), "=m"(CPUEXT), "=m"(ECS)
	 :
	 :"eax", "ebx", "ecx", "edx"
	 );
	return (CPUZ&SSE_MASK?1:0);
#endif
}
const float fonez[4] __attribute__((aligned(16))) = {1.0f, 1.0f, 1.0f, 1.0f};
void ArrayResize_SSE(Rgb *src, int src_len, Rgb *dest, int dest_len) __attribute__((noinline));
void ArrayResize_SSE(Rgb *src, int src_len, Rgb *dest, int dest_len)
{
#ifndef __APPLE__
	float ratio = (dest_len-1)/((float)src_len-1);

	for (int i = 0; i < src_len; i++)
		src[i].a = 1.0f;
	__asm __volatile(

	//setup:
	"	mov	%0,	%%esi\n"
	"	mov	%1,	%%edi\n"
	"	mov	%2,	%%ecx\n"
	"	movaps %5,	%%xmm7\n"
	"	xor	%%eax,	%%eax\n"
	
	//cycle1:
	"	.balign	16\n"
	"cycle1:\n"
	
	"	cvtsi2ss	%%eax,	%%xmm0\n"
	"	mulss	%4,	%%xmm0\n"
	"	cvttss2si	%%xmm0,	%%edx\n"
	"	movaps	%%xmm7,	%%xmm1\n"
	"	cvtsi2ss	%%edx,	%%xmm2\n"
	"	shl	$4,	%%edx\n"
	"	shl $4,	%%eax\n"
	"	subss	%%xmm2,	%%xmm0\n"
	"	movaps	(%%esi,%%eax),	%%xmm6\n"
	"	shr	$4,	%%eax\n"
	"	movaps	%%xmm0,	%%xmm2\n"
	"	shufps	$0,	%%xmm2,	%%xmm0\n"
	"	movaps	(%%edi,	%%edx),	%%xmm5\n"
	"	movaps	16(%%edi,	%%edx),	%%xmm4\n"

	"	subps	%%xmm0,	%%xmm1\n"
	
	// xmm0: multiplier for [xx+1]
	// xmm1: multiplier for [xx  ]
	"	mulps	%%xmm6,	%%xmm0\n"
	"	mulps	%%xmm6,	%%xmm1\n"
	"	addps	%%xmm4,	%%xmm0\n"
	"	addps	%%xmm5,	%%xmm1\n"
	
	"	movaps	%%xmm0,	16(%%edi,%%edx)\n"
	"	movaps	%%xmm1,	(%%edi, %%edx)\n"
	
	"	inc	%%eax\n"
	"	cmp	%%ecx,	%%eax\n"
	"	jb	cycle1\n"
	
	// setup for cycle 2:
	"	xor	%%eax,	%%eax\n"
	"	mov	%3,	%%ecx\n"
	"	shl	$4,	%%ecx\n"
	
	//cycle2:
	".balign 16\n"
	"cycle2:\n"
		
	"	movaps	(%%edi,	%%eax),	%%xmm0\n"
	"	movaps	%%xmm0,	%%xmm2\n"
	"	shufps	$0xff,	%%xmm0,	%%xmm2\n"
	"	divps	%%xmm2,	%%xmm0\n"
	"	movaps	%%xmm0,	(%%edi,	%%eax)\n"
	
	"	add	$16,	%%eax\n"
	"	cmp	%%ecx,	%%eax\n"
	"	jb	cycle2\n"
	:
	:"m"(src), "m"(dest), "m"(src_len), "m"(dest_len), "m"(ratio), "m"(*fonez)
	:"memory", "eax", "ecx", "edx", "esi", "edi"
	);
#endif
}

#else
#ifndef _MSC_VER
#error Unsupported compiler (must be GCC or MSVC)
#endif

bool SSESupported(void)
{
	int CPUZ=0, CPUEXT=0, ECS=0;
	 __asm {
	
		mov		eax,		1
		cpuid
		mov		CPUZ,		edx
		mov		eax,		0x80000000
		cpuid
		cmp		eax,		0x80000001
		jb		not_supported
	
		mov		eax,		0x80000001
		cpuid
		mov		CPUEXT,		edx
		mov		ECS,		1
		jmp		ende
	
	 not_supported:
		mov		ECS,		0
	 ende:
	
	 }
	return (CPUZ&SSE_MASK?1:0);
}
__declspec (align(16)) const float fonez[4] = {1.0f, 1.0f, 1.0f, 1.0f};
void ArrayResize_SSE(Rgb *src, int src_len, Rgb *dest, int dest_len)
{
	float ratio = (dest_len-1)/((float)src_len-1);

	for (int i = 0; i < src_len; i++)
		src[i].a = 1.0f;
	__asm {


	//setup:
		mov		esi,		src
		mov		edi,		dest
		mov		ecx,		src_len
		movaps		xmm7,		[fonez]
		xor		eax,		eax
	
	//cycle1:
	ALIGN	16
	cycle1:
	
		cvtsi2ss		xmm0,		eax
		mulss		xmm0,		ratio
		cvttss2si		edx,		xmm0
		movaps		xmm1,		xmm7
		cvtsi2ss		xmm2,		edx
		shl		edx,		4
		shl		eax,		4
		subss		xmm0,		xmm2
		movaps		xmm6,		[esi + eax]
		shr		eax,		4
		movaps		xmm2,		xmm0
		shufps		xmm0,		xmm2,		0
		movaps		xmm5,		[edi + edx]
		movaps		xmm4,		[edi + edx + 16]

		subps		xmm1,		xmm0
	
	// xmm0: multiplier for [xx+1]
	// xmm1: multiplier for [xx  ]
		mulps		xmm0,		xmm6
		mulps		xmm1,		xmm6
		addps		xmm0,		xmm4
		addps		xmm1,		xmm5
	
		movaps		[edi + edx + 16],		xmm0
		movaps		[edi + edx],		xmm1
	
		inc		eax
		cmp		eax,		ecx
		jb		cycle1
	
	// setup for cycle 2:
		xor		eax,		eax
		mov		ecx,		dest_len
		shl		ecx,		4
	
	//cycle2:
	ALIGN	16
	cycle2:
		
		movaps		xmm0,		[edi + eax]
		movaps		xmm2,		xmm0
		shufps		xmm2,		xmm0,		0xff
		divps		xmm0,		xmm2
		movaps		[edi + eax],		xmm0
	
		add		eax,		16
		cmp		eax,		ecx
		jb		cycle2
	
	}
}


#endif


void ArrayResize(Rgb *src, int src_len, Rgb *dest, int dest_len)
{
	if (src_len == dest_len) {
		memcpy(dest, src, src_len * sizeof(Rgb));
		return;
	}
	for (int i = 0; i < dest_len+1; i++) {
		dest[i].a = dest[i].r = dest[i].g = dest[i].b = 0.0f;
	}
	static int UseSSE = -1;
	if (UseSSE == -1) {
		UseSSE = preferences.no_sse ? 0 : SSESupported();
	}
	if (UseSSE)
		ArrayResize_SSE(src, src_len, dest, dest_len);
	else
		ArrayResize_x87(src, src_len, dest, dest_len);
}
