#include <math.h>
#include <string.h>
#include <algorithm>
#include "abstract_sphere.h"

double get_mult(int a[] )
{
	int c = 0;
	for (int i = 0; i < 4; i++)
		for (int j = i+1; j < 4; j++)
			c += (a[i] > a[j]);
	if (c % 2) return -1;
	else return +1;
}

class Matrix4x4 {
	double a[4][4];
public:
	Matrix4x4() { memset(a, 0, sizeof(a)); }
	void set(int i, int j, double x) { a[i][j] = x; }
	double det(void)
	{
		double sum = 0.0;
		int ord[4] = {0, 1, 2, 3};
		do {
			sum += get_mult(ord) * a[0][ord[0]] * a[1][ord[1]] * a[2][ord[2]] * a[3][ord[3]];
		} while (std::next_permutation(ord, ord+4));
		return sum;
	}
};

////////////////////////////////////////////////////
/// @class Circle
////////////////////////////////////////////////////

double face(const Vector& a, const Vector &b, const Vector &c)
{
	return a[0] * b[1] + b[0] * c[1] + c[0] * a[1] 
	      -c[0] * b[1] - b[0] * a[1] - a[0] * c[1];
}

bool Circle::construct(Point points[])
{
	Vector a(points[0]), b(points[1]), c(points[2]);
	a[2] = b[2] = c[2] = 0;
	if (fabs(face(a,b,c)) < 1e-9) return false;

	Vector p = (a+b) * 0.5;
	Vector q = (a+c) * 0.5;
	Vector u = (b-a);
	Vector v = (c-a);
	u.rot90();
	v.rot90();
	v.scale(-1);
	double Det = u.x * v.y - u.y * v.x;
	Vector h = q - p;
	double alpha = (h.x * v.y - h.y * v.x) / Det;

	p += u * alpha;

	center = p;

	radius = center.distto(a);

	return true;
}

bool Circle::is_inside(const Point& p)
{
	Vector x(p);
	x[2] = 0;
	return x.distto(center) < radius - 1e-9;
}

void Circle::print(void) const
{
	printf("Radius = %.3lf ", radius);
	center.print();
}

////////////////////////////////////////////////////
/// @class Sphere
////////////////////////////////////////////////////

bool Sphere::construct(Point points[])
{
	Vector a(points[0]), b(points[1]), c(points[2]), d(points[3]);
	Vector u = (b-a), v = (c-a);
	Vector normal = u ^ v;

	double Q = - (a * normal);

	if (fabs(Q + (d * normal)) < 1e-9) return false;

	Vector all[4] = {a,b,c,d};

	Matrix4x4 A;

	for (int i = 0; i < 4; i++) {
		A.set(i, 3, 1);
		for (int j = 0; j < 3; j++) A.set(i, j, all[i][j]);
	}
	double alpha = A.det();

	for (int i = 0; i < 4; i++) {
		A.set(i, 0, all[i].lengthSqr());
		A.set(i, 3, 1);
	}

	int indices[3][3] = { {1, 2}, {0, 2}, {0, 1} };
	double D[3];

	for (int i = 0; i < 3; i++) {
		for (int j = 0;  j < 4; j++) {
			A.set(j, 1, all[j][indices[i][0]]);
			A.set(j, 2, all[j][indices[i][1]]);
		}
		D[i] = A.det();
	}
	D[1] = - D[1];
	alpha = 1 / (2.0 *alpha);
	center = Vector(D[0] * alpha, D[1] * alpha, D[2] * alpha);
	radius = center.distto(a);
	
	return true;
}

bool Sphere::is_inside(const Point& p)
{
	Vector x(p);
	return x.distto(center) < radius - 1e-9;
}

