#include <assert.h>
#include <string>
#include <list>
#include "complex.h"
using std::string;
using std::list;
using std::endl;

/// Adds a simplex to the current complex
/// Checks if the simplex can be added
bool Complex::add(const Simplex& a)
{
	if (contains(a)) return false;
	if (a.get_dim() == 0) {
		data.push_back(a);
		return true;
	}
	Simplex copy = a;
	copy.iter_start(true);
	Simplex test;
	while (copy.iter(test))
	{
		if (!contains(test)) {
			return false;
		}
	}
	data.push_back(a);
	return true;
}

/// Does the current complex contain the simplex c?
bool Complex::contains(const Simplex& c)
{
	for (ComplexData::iterator iter = data.begin(); iter != data.end(); iter++) {
		if (iter->get_dim() != c.get_dim())
			continue;
		if ( (*iter) == c )
			return true;
	}

	return false;
}

int Complex::size(void) const
{
	return (int)data.size();
}

const Simplex& Complex::operator [] (int index) const
{
	assert(index >= 0 && index < data.size());
	return data[index];
}

Simplex& Complex::operator [] (int index)
{
	assert(index >= 0 && index < data.size());
	return data[index];
}

/// Input / output

std::istream& operator >> (std::istream& in, Complex& c)
{
	string s;
	in >> s;
	in >> s;
	in >> s;
	unsigned d;
	int r = sscanf(s.c_str(), "num=\"%u\">", &d);
	assert( 1 == r );

	c.data.clear();
	for (int i = 0; i < d; i++) {
		Simplex sim;
		in >> sim;
		bool res = c.add(sim);
		if (!res) {
			cout << "Error on adding the following simplex: " << sim << endl;
		}
	}
	in >> s;
	in >> s;
	return in;
}

std::ostream& operator << (std::ostream& out, Complex& c)
{
	out << "\t\t\t<faceSet>" << endl;
	out << "\t\t\t\t<faces num=\"" << c.size() << "\">" << endl;
	for (int i = 0; i < c.size(); i++) {
		out << "\t\t\t\t\t" << c[i] << endl;
	}
	out << "\t\t\t\t</faces>" << endl;
	out << "\t\t\t</faceSet>";
	return out;
}

