#ifndef __GEOMETRY_H__
#define __GEOMETRY_H__

#include <iostream>
#include <vector>
#include <string>
#include "simplex.h"
#include "complex.h"
#include "pointset.h"


class Geometry {
	PointSet *points;
	Complex *data;
	std::string name;
	//
public:
	Geometry();
	Geometry(PointSet*, Complex *);
	~Geometry();
	void set_complex(Complex *);
	void set_name(std::string);
	PointSet* get_points(void) const;
	Complex* get_data(void) const;
	//
	friend std::istream& operator >> (std::istream&, Geometry&);
	friend std::ostream& operator << (std::ostream&, Geometry&);
	//
	static string begin_jvx();
	static string end_jvx();

	//Advanced stuff
	/**
	 * Performs delaunay triangulation
	 * @param pointset - the points to triangulate
	 * @param dim      - the dimension to work with
	 *                   (useful values = {2, 3})
	*/
	bool delaunay_triangulation(PointSet *pointset, int dim);
	void recurse(int dim, int lev, bool used[], int last);
};

#endif
