#include <assert.h>
#include <string>
#include "pointset.h"
using std::vector;
using std::endl;
using std::string;

/*
	@Class Point
*/
Point::Point() {}

Point::Point(double aa[], int n)
{
	for (int i = 0; i < n; i++)
		a[i] = aa[i];
}

const double& Point::operator [] (int index) const
{
	assert (index >= 0 && index < MAXDIM);
	return a[index];
}

double& Point::operator [] (int index)
{
	assert (index >= 0 && index < MAXDIM);
	return a[index];
}

/*
	@Class PointSet
*/

int PointSet::dim = 0;

void PointSet::add(const Point &a)
{
	data.push_back(a);
}

int PointSet::size()
{
	return (int) data.size();
}

const Point& PointSet::operator [] (int index) const
{
	assert (index >= 0 && index < data.size());
	return data[index];
}

Point& PointSet::operator [] (int index) 
{
	assert (index >= 0 && index < data.size());
	return data[index];
}

std::istream& operator >> (istream& in, PointSet& p)
{
	string s;
	in >> s;
	in >> s;

	int dim = -1;
	sscanf(s.c_str(), "dim=\"%d\"", &dim);

	p.dim = dim;

	in >> s;
	in >> s;
	int N = -1;
	sscanf(s.c_str(), "num=\"%d\"", &N);

	for (int i = 0; i < N; i++)
	{
		Point pp;
		for (int j = 0; j < dim; j++)
		{
			in >> s;
			if ( s.substr(0, 3) == "<p>" )
				s = s.substr(3);
			if (s.find("</p>") != -1)
				s = s.substr(0, s.find("</p>"));
			double x;
			if (1 == sscanf(s.c_str(), "%lf", &x))
				pp[j] = x;
		}
		p.add(pp);
	}

	in >> s;
	in >> s;

	return in;
}

std::ostream& operator << (ostream& out, PointSet& p)
{
	out << "\t\t\t<pointSet dim=\"" << p.dim << "\">" << endl;
	out << "\t\t\t\t<points num=\"" << p.size() << "\">" << endl;
	for (int i = 0; i < p.size(); i++)
	{
		out << "\t\t\t\t\t<p>";
		for (int v = 0; v < p.dim; v++)
		{
			if (v) out << " ";
			out << p.data[i][v];
		}
		out << "</p>" << endl;
	}
	out << "\t\t\t\t</points>" << endl << "\t\t\t</pointSet>";

	return out;
}
